/* ############################# MPC License ############################## */
/* # Wed Nov 19 15:19:19 CET 2008                                         # */
/* # Copyright or (C) or Copr. Commissariat a l'Energie Atomique          # */
/* #                                                                      # */
/* # IDDN.FR.001.230040.000.S.P.2007.000.10000                            # */
/* # This file is part of the MPC Runtime.                                # */
/* #                                                                      # */
/* # This software is governed by the CeCILL-C license under French law   # */
/* # and abiding by the rules of distribution of free software.  You can  # */
/* # use, modify and/ or redistribute the software under the terms of     # */
/* # the CeCILL-C license as circulated by CEA, CNRS and INRIA at the     # */
/* # following URL http://www.cecill.info.                                # */
/* #                                                                      # */
/* # The fact that you are presently reading this means that you have     # */
/* # had knowledge of the CeCILL-C license and that you accept its        # */
/* # terms.                                                               # */
/* #                                                                      # */
/* # Authors:                                                             # */
/* #   - PERACHE Marc marc.perache@cea.fr                                 # */
/* #                                                                      # */
/* ######################################################################## */
#include "sctk_asm.h"
#include "sctk_pthread.h"
#include "sctk_kernel_thread.h"


#ifndef SCTK_KERNEL_THREAD_USE_TLS
int
kthread_key_create (kthread_key_t * key, void (*destr_function) (void *))
{
  return pthread_key_create ((pthread_key_t *) key, destr_function);
}

int
kthread_key_delete (kthread_key_t key)
{
  pthread_key_t keya;
  pthread_key_t *keyp;
  keyp = (pthread_key_t *) & key;
  keya = *keyp;
  return pthread_key_delete (keya);
}

int
kthread_setspecific (kthread_key_t key, const void *pointer)
{
  pthread_key_t keya;
  pthread_key_t *keyp;
  keyp = (pthread_key_t *) & key;
  keya = *keyp;
  return pthread_setspecific (keya, pointer);
}

void *
kthread_getspecific (kthread_key_t key)
{
  pthread_key_t keya;
  pthread_key_t *keyp;
  keyp = (pthread_key_t *) & key;
  keya = *keyp;
  return pthread_getspecific (keya);
}
#endif


#define kthread_stack_size (1*1024*1024)

typedef struct
{
  void *(*start_routine) (void *);
  void *arg;
  volatile int started;
} kthread_create_start_t;
static void *
kthread_create_start_routine (void *t_arg)
{
  kthread_create_start_t *tmp;
  void *(*start_routine) (void *);
  void *arg;

  tmp = (kthread_create_start_t *) t_arg;
  start_routine = tmp->start_routine;
  arg = tmp->arg;
  tmp->started = 1;

  return start_routine (arg);
}

int
kthread_create (kthread_t * thread, void *(*start_routine) (void *),
		void *arg)
{
  pthread_attr_t attr;
  int res;
  kthread_create_start_t tmp;

  pthread_attr_init (&attr);
  pthread_attr_setscope (&attr, PTHREAD_SCOPE_SYSTEM);
#ifdef PTHREAD_STACK_MIN
  if (PTHREAD_STACK_MIN > kthread_stack_size)
    {
      pthread_attr_setstacksize (&attr, PTHREAD_STACK_MIN);
    }
  else
    {
      pthread_attr_setstacksize (&attr, kthread_stack_size);
    }
#else
  pthread_attr_setstacksize (&attr, kthread_stack_size);
#endif
  assume (sizeof (kthread_t) == sizeof (pthread_t));

  tmp.started = 0;
  tmp.arg = arg;
  tmp.start_routine = start_routine;

  res =
    pthread_create ((pthread_t *) thread, &attr, kthread_create_start_routine,
		    &tmp);
  pthread_attr_destroy (&attr);
  while (tmp.started != 1)
    {
      sched_yield ();
    }
  return res;
}

int
kthread_join (kthread_t th, void **thread_return)
{
  return pthread_join ((pthread_t) th, thread_return);
}


int
kthread_kill (kthread_t th, int val)
{
  return pthread_kill ((pthread_t) th, val);
}


kthread_t
kthread_self ()
{
  return (kthread_t) pthread_self ();
}

int
kthread_sigmask (int how, const sigset_t * newmask, sigset_t * oldmask)
{
  return pthread_sigmask (how, newmask, oldmask);
}

void
kthread_exit (void *retval)
{
  pthread_exit (retval);
}
