
#include <math.h>
#include "mpc.h"
#include <stdio.h>
#include <stdlib.h>
#include "test.h"
#include "gcomm.h"

int
main (int argc, char **argv)
{
  int count, errcnt = 0, gerr = 0, size, rank;
  MPC_Comm comm;

  MPC_Comm comms[10];
  int ncomm, i, world_rank;

  MPC_Init (&argc, &argv);
  MPC_Comm_rank (MPC_COMM_WORLD, &world_rank);

/* First tests */
  MakeComms (comms, 10, &ncomm, 0);
  for (i = 0; i < ncomm; i++)
    {
      if (world_rank == 0)
	printf ("Testing with communicator %d\n", i);
      comm = comms[i];


      MPC_Comm_size (comm, &size);
      MPC_Comm_rank (comm, &rank);
      count = 5;

/* Test sum */
      if (world_rank == 0)
	printf ("Testing MPC_SUM...\n");



      {
	int *in, *out, *sol;
	int i, fnderr = 0;
	in = (int *) malloc (count * sizeof (int));
	out = (int *) malloc (count * sizeof (int));
	sol = (int *) malloc (count * sizeof (int));
	for (i = 0; i < count; i++)
	  {
	    *(in + i) = i;
	    *(sol + i) = i * size;
	    *(out + i) = 0;
	  }
	MPC_Allreduce (in, out, count, MPC_INT, MPC_SUM, comm);
	for (i = 0; i < count; i++)
	  {
	    if (*(out + i) != *(sol + i))
	      {
		errcnt++;
		fnderr++;
	      }
	  }
	if (fnderr)
	  fprintf (stderr,
		   "(%d) Error for type MPC_INT and op MPC_SUM\n",
		   world_rank);
	free (in);
	free (out);
	free (sol);
      }


      {
	long *in, *out, *sol;
	int i, fnderr = 0;
	in = (long *) malloc (count * sizeof (long));
	out = (long *) malloc (count * sizeof (long));
	sol = (long *) malloc (count * sizeof (long));
	for (i = 0; i < count; i++)
	  {
	    *(in + i) = i;
	    *(sol + i) = i * size;
	    *(out + i) = 0;
	  }
	MPC_Allreduce (in, out, count, MPC_LONG, MPC_SUM, comm);
	for (i = 0; i < count; i++)
	  {
	    if (*(out + i) != *(sol + i))
	      {
		errcnt++;
		fnderr++;
	      }
	  }
	if (fnderr)
	  fprintf (stderr,
		   "(%d) Error for type MPC_LONG and op MPC_SUM\n",
		   world_rank);
	free (in);
	free (out);
	free (sol);
      }


      {
	short *in, *out, *sol;
	int i, fnderr = 0;
	in = (short *) malloc (count * sizeof (short));
	out = (short *) malloc (count * sizeof (short));
	sol = (short *) malloc (count * sizeof (short));
	for (i = 0; i < count; i++)
	  {
	    *(in + i) = i;
	    *(sol + i) = i * size;
	    *(out + i) = 0;
	  }
	MPC_Allreduce (in, out, count, MPC_SHORT, MPC_SUM, comm);
	for (i = 0; i < count; i++)
	  {
	    if (*(out + i) != *(sol + i))
	      {
		errcnt++;
		fnderr++;
	      }
	  }
	if (fnderr)
	  fprintf (stderr,
		   "(%d) Error for type MPC_SHORT and op MPC_SUM\n",
		   world_rank);
	free (in);
	free (out);
	free (sol);
      }


      {
	unsigned short *in, *out, *sol;
	int i, fnderr = 0;
	in = (unsigned short *) malloc (count * sizeof (unsigned short));
	out = (unsigned short *) malloc (count * sizeof (unsigned short));
	sol = (unsigned short *) malloc (count * sizeof (unsigned short));
	for (i = 0; i < count; i++)
	  {
	    *(in + i) = i;
	    *(sol + i) = i * size;
	    *(out + i) = 0;
	  }
	MPC_Allreduce (in, out, count, MPC_UNSIGNED_SHORT, MPC_SUM, comm);
	for (i = 0; i < count; i++)
	  {
	    if (*(out + i) != *(sol + i))
	      {
		errcnt++;
		fnderr++;
	      }
	  }
	if (fnderr)
	  fprintf (stderr,
		   "(%d) Error for type MPC_UNSIGNED_SHORT and op MPC_SUM\n",
		   world_rank);
	free (in);
	free (out);
	free (sol);
      }


      {
	unsigned *in, *out, *sol;
	int i, fnderr = 0;
	in = (unsigned *) malloc (count * sizeof (unsigned));
	out = (unsigned *) malloc (count * sizeof (unsigned));
	sol = (unsigned *) malloc (count * sizeof (unsigned));
	for (i = 0; i < count; i++)
	  {
	    *(in + i) = i;
	    *(sol + i) = i * size;
	    *(out + i) = 0;
	  }
	MPC_Allreduce (in, out, count, MPC_UNSIGNED, MPC_SUM, comm);
	for (i = 0; i < count; i++)
	  {
	    if (*(out + i) != *(sol + i))
	      {
		errcnt++;
		fnderr++;
	      }
	  }
	if (fnderr)
	  fprintf (stderr,
		   "(%d) Error for type MPC_UNSIGNED and op MPC_SUM\n",
		   world_rank);
	free (in);
	free (out);
	free (sol);
      }


      {
	unsigned long *in, *out, *sol;
	int i, fnderr = 0;
	in = (unsigned long *) malloc (count * sizeof (unsigned long));
	out = (unsigned long *) malloc (count * sizeof (unsigned long));
	sol = (unsigned long *) malloc (count * sizeof (unsigned long));
	for (i = 0; i < count; i++)
	  {
	    *(in + i) = i;
	    *(sol + i) = i * size;
	    *(out + i) = 0;
	  }
	MPC_Allreduce (in, out, count, MPC_UNSIGNED_LONG, MPC_SUM, comm);
	for (i = 0; i < count; i++)
	  {
	    if (*(out + i) != *(sol + i))
	      {
		errcnt++;
		fnderr++;
	      }
	  }
	if (fnderr)
	  fprintf (stderr,
		   "(%d) Error for type MPC_UNSIGNED_LONG and op MPC_SUM\n",
		   world_rank);
	free (in);
	free (out);
	free (sol);
      }


      {
	float *in, *out, *sol;
	int i, fnderr = 0;
	in = (float *) malloc (count * sizeof (float));
	out = (float *) malloc (count * sizeof (float));
	sol = (float *) malloc (count * sizeof (float));
	for (i = 0; i < count; i++)
	  {
	    *(in + i) = i;
	    *(sol + i) = i * size;
	    *(out + i) = 0;
	  }
	MPC_Allreduce (in, out, count, MPC_FLOAT, MPC_SUM, comm);
	for (i = 0; i < count; i++)
	  {
	    if (*(out + i) != *(sol + i))
	      {
		errcnt++;
		fnderr++;
	      }
	  }
	if (fnderr)
	  fprintf (stderr,
		   "(%d) Error for type MPC_FLOAT and op MPC_SUM\n",
		   world_rank);
	free (in);
	free (out);
	free (sol);
      }


      {
	double *in, *out, *sol;
	int i, fnderr = 0;
	in = (double *) malloc (count * sizeof (double));
	out = (double *) malloc (count * sizeof (double));
	sol = (double *) malloc (count * sizeof (double));
	for (i = 0; i < count; i++)
	  {
	    *(in + i) = i;
	    *(sol + i) = i * size;
	    *(out + i) = 0;
	  }
	MPC_Allreduce (in, out, count, MPC_DOUBLE, MPC_SUM, comm);
	for (i = 0; i < count; i++)
	  {
	    if (*(out + i) != *(sol + i))
	      {
		errcnt++;
		fnderr++;
	      }
	  }
	if (fnderr)
	  fprintf (stderr,
		   "(%d) Error for type MPC_DOUBLE and op MPC_SUM\n",
		   world_rank);
	free (in);
	free (out);
	free (sol);
      }


      gerr += errcnt;
      if (errcnt > 0)
	printf ("Found %d errors on %d for MPC_SUM\n", errcnt, rank);
      errcnt = 0;

/* Test product */
      if (world_rank == 0)
	printf ("Testing MPC_PROD...\n");

      {
	int *in, *out, *sol;
	int i, fnderr = 0;
	in = (int *) malloc (count * sizeof (int));
	out = (int *) malloc (count * sizeof (int));
	sol = (int *) malloc (count * sizeof (int));
	for (i = 0; i < count; i++)
	  {
	    *(in + i) = i;
	    *(sol + i) =
	      (i > 0) ? (int) (pow ((double) (i), (double) size) + 0.1) : 0;
	    *(out + i) = 0;
	  }
	MPC_Allreduce (in, out, count, MPC_INT, MPC_PROD, comm);
	for (i = 0; i < count; i++)
	  {
	    if (*(out + i) != *(sol + i))
	      {
		errcnt++;
		fnderr++;
	      }
	  }
	if (fnderr)
	  fprintf (stderr,
		   "(%d) Error for type MPC_INT and op MPC_PROD\n",
		   world_rank);
	free (in);
	free (out);
	free (sol);
      }


      {
	long *in, *out, *sol;
	int i, fnderr = 0;
	in = (long *) malloc (count * sizeof (long));
	out = (long *) malloc (count * sizeof (long));
	sol = (long *) malloc (count * sizeof (long));
	for (i = 0; i < count; i++)
	  {
	    *(in + i) = i;
	    *(sol + i) =
	      (i > 0) ? (int) (pow ((double) (i), (double) size) + 0.1) : 0;
	    *(out + i) = 0;
	  }
	MPC_Allreduce (in, out, count, MPC_LONG, MPC_PROD, comm);
	for (i = 0; i < count; i++)
	  {
	    if (*(out + i) != *(sol + i))
	      {
		errcnt++;
		fnderr++;
	      }
	  }
	if (fnderr)
	  fprintf (stderr,
		   "(%d) Error for type MPC_LONG and op MPC_PROD\n",
		   world_rank);
	free (in);
	free (out);
	free (sol);
      }


      {
	short *in, *out, *sol;
	int i, fnderr = 0;
	in = (short *) malloc (count * sizeof (short));
	out = (short *) malloc (count * sizeof (short));
	sol = (short *) malloc (count * sizeof (short));
	for (i = 0; i < count; i++)
	  {
	    *(in + i) = i;
	    *(sol + i) =
	      (i > 0) ? (int) (pow ((double) (i), (double) size) + 0.1) : 0;
	    *(out + i) = 0;
	  }
	MPC_Allreduce (in, out, count, MPC_SHORT, MPC_PROD, comm);
	for (i = 0; i < count; i++)
	  {
	    if (*(out + i) != *(sol + i))
	      {
		errcnt++;
		fnderr++;
	      }
	  }
	if (fnderr)
	  fprintf (stderr,
		   "(%d) Error for type MPC_SHORT and op MPC_PROD\n",
		   world_rank);
	free (in);
	free (out);
	free (sol);
      }


      {
	unsigned short *in, *out, *sol;
	int i, fnderr = 0;
	in = (unsigned short *) malloc (count * sizeof (unsigned short));
	out = (unsigned short *) malloc (count * sizeof (unsigned short));
	sol = (unsigned short *) malloc (count * sizeof (unsigned short));
	for (i = 0; i < count; i++)
	  {
	    *(in + i) = i;
	    *(sol + i) =
	      (i > 0) ? (int) (pow ((double) (i), (double) size) + 0.1) : 0;
	    *(out + i) = 0;
	  }
	MPC_Allreduce (in, out, count, MPC_UNSIGNED_SHORT, MPC_PROD, comm);
	for (i = 0; i < count; i++)
	  {
	    if (*(out + i) != *(sol + i))
	      {
		errcnt++;
		fnderr++;
	      }
	  }
	if (fnderr)
	  fprintf (stderr,
		   "(%d) Error for type MPC_UNSIGNED_SHORT and op MPC_PROD\n",
		   world_rank);
	free (in);
	free (out);
	free (sol);
      }


      {
	unsigned *in, *out, *sol;
	int i, fnderr = 0;
	in = (unsigned *) malloc (count * sizeof (unsigned));
	out = (unsigned *) malloc (count * sizeof (unsigned));
	sol = (unsigned *) malloc (count * sizeof (unsigned));
	for (i = 0; i < count; i++)
	  {
	    *(in + i) = i;
	    *(sol + i) =
	      (i > 0) ? (int) (pow ((double) (i), (double) size) + 0.1) : 0;
	    *(out + i) = 0;
	  }
	MPC_Allreduce (in, out, count, MPC_UNSIGNED, MPC_PROD, comm);
	for (i = 0; i < count; i++)
	  {
	    if (*(out + i) != *(sol + i))
	      {
		errcnt++;
		fnderr++;
	      }
	  }
	if (fnderr)
	  fprintf (stderr,
		   "(%d) Error for type MPC_UNSIGNED and op MPC_PROD\n",
		   world_rank);
	free (in);
	free (out);
	free (sol);
      }


      {
	unsigned long *in, *out, *sol;
	int i, fnderr = 0;
	in = (unsigned long *) malloc (count * sizeof (unsigned long));
	out = (unsigned long *) malloc (count * sizeof (unsigned long));
	sol = (unsigned long *) malloc (count * sizeof (unsigned long));
	for (i = 0; i < count; i++)
	  {
	    *(in + i) = i;
	    *(sol + i) =
	      (i > 0) ? (int) (pow ((double) (i), (double) size) + 0.1) : 0;
	    *(out + i) = 0;
	  }
	MPC_Allreduce (in, out, count, MPC_UNSIGNED_LONG, MPC_PROD, comm);
	for (i = 0; i < count; i++)
	  {
	    if (*(out + i) != *(sol + i))
	      {
		errcnt++;
		fnderr++;
	      }
	  }
	if (fnderr)
	  fprintf (stderr,
		   "(%d) Error for type MPC_UNSIGNED_LONG and op MPC_PROD\n",
		   world_rank);
	free (in);
	free (out);
	free (sol);
      }


      {
	float *in, *out, *sol;
	int i, fnderr = 0;
	in = (float *) malloc (count * sizeof (float));
	out = (float *) malloc (count * sizeof (float));
	sol = (float *) malloc (count * sizeof (float));
	for (i = 0; i < count; i++)
	  {
	    *(in + i) = i;
	    *(sol + i) =
	      (i > 0) ? (int) (pow ((double) (i), (double) size) + 0.1) : 0;
	    *(out + i) = 0;
	  }
	MPC_Allreduce (in, out, count, MPC_FLOAT, MPC_PROD, comm);
	for (i = 0; i < count; i++)
	  {
	    if (*(out + i) != *(sol + i))
	      {
		errcnt++;
		fnderr++;
	      }
	  }
	if (fnderr)
	  fprintf (stderr,
		   "(%d) Error for type MPC_FLOAT and op MPC_PROD\n",
		   world_rank);
	free (in);
	free (out);
	free (sol);
      }


      {
	double *in, *out, *sol;
	int i, fnderr = 0;
	in = (double *) malloc (count * sizeof (double));
	out = (double *) malloc (count * sizeof (double));
	sol = (double *) malloc (count * sizeof (double));
	for (i = 0; i < count; i++)
	  {
	    *(in + i) = i;
	    *(sol + i) =
	      (i > 0) ? (int) (pow ((double) (i), (double) size) + 0.1) : 0;
	    *(out + i) = 0;
	  }
	MPC_Allreduce (in, out, count, MPC_DOUBLE, MPC_PROD, comm);
	for (i = 0; i < count; i++)
	  {
	    if (*(out + i) != *(sol + i))
	      {
		errcnt++;
		fnderr++;
	      }
	  }
	if (fnderr)
	  fprintf (stderr,
		   "(%d) Error for type MPC_DOUBLE and op MPC_PROD\n",
		   world_rank);
	free (in);
	free (out);
	free (sol);
      }


      gerr += errcnt;
      if (errcnt > 0)
	printf ("Found %d errors on %d for MPC_PROD\n", errcnt, rank);
      errcnt = 0;

/* Test max */
      if (world_rank == 0)
	printf ("Testing MPC_MAX...\n");

      {
	int *in, *out, *sol;
	int i, fnderr = 0;
	in = (int *) malloc (count * sizeof (int));
	out = (int *) malloc (count * sizeof (int));
	sol = (int *) malloc (count * sizeof (int));
	for (i = 0; i < count; i++)
	  {
	    *(in + i) = (rank + i);
	    *(sol + i) = (size - 1 + i);
	    *(out + i) = 0;
	  }
	MPC_Allreduce (in, out, count, MPC_INT, MPC_MAX, comm);
	for (i = 0; i < count; i++)
	  {
	    if (*(out + i) != *(sol + i))
	      {
		errcnt++;
		fnderr++;
	      }
	  }
	if (fnderr)
	  fprintf (stderr,
		   "(%d) Error for type MPC_INT and op MPC_MAX\n",
		   world_rank);
	free (in);
	free (out);
	free (sol);
      }


      {
	long *in, *out, *sol;
	int i, fnderr = 0;
	in = (long *) malloc (count * sizeof (long));
	out = (long *) malloc (count * sizeof (long));
	sol = (long *) malloc (count * sizeof (long));
	for (i = 0; i < count; i++)
	  {
	    *(in + i) = (rank + i);
	    *(sol + i) = (size - 1 + i);
	    *(out + i) = 0;
	  }
	MPC_Allreduce (in, out, count, MPC_LONG, MPC_MAX, comm);
	for (i = 0; i < count; i++)
	  {
	    if (*(out + i) != *(sol + i))
	      {
		errcnt++;
		fnderr++;
	      }
	  }
	if (fnderr)
	  fprintf (stderr,
		   "(%d) Error for type MPC_LONG and op MPC_MAX\n",
		   world_rank);
	free (in);
	free (out);
	free (sol);
      }


      {
	short *in, *out, *sol;
	int i, fnderr = 0;
	in = (short *) malloc (count * sizeof (short));
	out = (short *) malloc (count * sizeof (short));
	sol = (short *) malloc (count * sizeof (short));
	for (i = 0; i < count; i++)
	  {
	    *(in + i) = (rank + i);
	    *(sol + i) = (size - 1 + i);
	    *(out + i) = 0;
	  }
	MPC_Allreduce (in, out, count, MPC_SHORT, MPC_MAX, comm);
	for (i = 0; i < count; i++)
	  {
	    if (*(out + i) != *(sol + i))
	      {
		errcnt++;
		fnderr++;
	      }
	  }
	if (fnderr)
	  fprintf (stderr,
		   "(%d) Error for type MPC_SHORT and op MPC_MAX\n",
		   world_rank);
	free (in);
	free (out);
	free (sol);
      }


      {
	unsigned short *in, *out, *sol;
	int i, fnderr = 0;
	in = (unsigned short *) malloc (count * sizeof (unsigned short));
	out = (unsigned short *) malloc (count * sizeof (unsigned short));
	sol = (unsigned short *) malloc (count * sizeof (unsigned short));
	for (i = 0; i < count; i++)
	  {
	    *(in + i) = (rank + i);
	    *(sol + i) = (size - 1 + i);
	    *(out + i) = 0;
	  }
	MPC_Allreduce (in, out, count, MPC_UNSIGNED_SHORT, MPC_MAX, comm);
	for (i = 0; i < count; i++)
	  {
	    if (*(out + i) != *(sol + i))
	      {
		errcnt++;
		fnderr++;
	      }
	  }
	if (fnderr)
	  fprintf (stderr,
		   "(%d) Error for type MPC_UNSIGNED_SHORT and op MPC_MAX\n",
		   world_rank);
	free (in);
	free (out);
	free (sol);
      }


      {
	unsigned *in, *out, *sol;
	int i, fnderr = 0;
	in = (unsigned *) malloc (count * sizeof (unsigned));
	out = (unsigned *) malloc (count * sizeof (unsigned));
	sol = (unsigned *) malloc (count * sizeof (unsigned));
	for (i = 0; i < count; i++)
	  {
	    *(in + i) = (rank + i);
	    *(sol + i) = (size - 1 + i);
	    *(out + i) = 0;
	  }
	MPC_Allreduce (in, out, count, MPC_UNSIGNED, MPC_MAX, comm);
	for (i = 0; i < count; i++)
	  {
	    if (*(out + i) != *(sol + i))
	      {
		errcnt++;
		fnderr++;
	      }
	  }
	if (fnderr)
	  fprintf (stderr,
		   "(%d) Error for type MPC_UNSIGNED and op MPC_MAX\n",
		   world_rank);
	free (in);
	free (out);
	free (sol);
      }


      {
	unsigned long *in, *out, *sol;
	int i, fnderr = 0;
	in = (unsigned long *) malloc (count * sizeof (unsigned long));
	out = (unsigned long *) malloc (count * sizeof (unsigned long));
	sol = (unsigned long *) malloc (count * sizeof (unsigned long));
	for (i = 0; i < count; i++)
	  {
	    *(in + i) = (rank + i);
	    *(sol + i) = (size - 1 + i);
	    *(out + i) = 0;
	  }
	MPC_Allreduce (in, out, count, MPC_UNSIGNED_LONG, MPC_MAX, comm);
	for (i = 0; i < count; i++)
	  {
	    if (*(out + i) != *(sol + i))
	      {
		errcnt++;
		fnderr++;
	      }
	  }
	if (fnderr)
	  fprintf (stderr,
		   "(%d) Error for type MPC_UNSIGNED_LONG and op MPC_MAX\n",
		   world_rank);
	free (in);
	free (out);
	free (sol);
      }


      {
	float *in, *out, *sol;
	int i, fnderr = 0;
	in = (float *) malloc (count * sizeof (float));
	out = (float *) malloc (count * sizeof (float));
	sol = (float *) malloc (count * sizeof (float));
	for (i = 0; i < count; i++)
	  {
	    *(in + i) = (rank + i);
	    *(sol + i) = (size - 1 + i);
	    *(out + i) = 0;
	  }
	MPC_Allreduce (in, out, count, MPC_FLOAT, MPC_MAX, comm);
	for (i = 0; i < count; i++)
	  {
	    if (*(out + i) != *(sol + i))
	      {
		errcnt++;
		fnderr++;
	      }
	  }
	if (fnderr)
	  fprintf (stderr,
		   "(%d) Error for type MPC_FLOAT and op MPC_MAX\n",
		   world_rank);
	free (in);
	free (out);
	free (sol);
      }


      {
	double *in, *out, *sol;
	int i, fnderr = 0;
	in = (double *) malloc (count * sizeof (double));
	out = (double *) malloc (count * sizeof (double));
	sol = (double *) malloc (count * sizeof (double));
	for (i = 0; i < count; i++)
	  {
	    *(in + i) = (rank + i);
	    *(sol + i) = (size - 1 + i);
	    *(out + i) = 0;
	  }
	MPC_Allreduce (in, out, count, MPC_DOUBLE, MPC_MAX, comm);
	for (i = 0; i < count; i++)
	  {
	    if (*(out + i) != *(sol + i))
	      {
		errcnt++;
		fnderr++;
	      }
	  }
	if (fnderr)
	  fprintf (stderr,
		   "(%d) Error for type MPC_DOUBLE and op MPC_MAX\n",
		   world_rank);
	free (in);
	free (out);
	free (sol);
      }


      gerr += errcnt;
      if (errcnt > 0)
	printf ("Found %d errors on %d for MPC_MAX\n", errcnt, rank);
      errcnt = 0;

/* Test min */
      if (world_rank == 0)
	printf ("Testing MPC_MIN...\n");

      {
	int *in, *out, *sol;
	int i, fnderr = 0;
	in = (int *) malloc (count * sizeof (int));
	out = (int *) malloc (count * sizeof (int));
	sol = (int *) malloc (count * sizeof (int));
	for (i = 0; i < count; i++)
	  {
	    *(in + i) = (rank + i);
	    *(sol + i) = i;
	    *(out + i) = 0;
	  }
	MPC_Allreduce (in, out, count, MPC_INT, MPC_MIN, comm);
	for (i = 0; i < count; i++)
	  {
	    if (*(out + i) != *(sol + i))
	      {
		errcnt++;
		fnderr++;
	      }
	  }
	if (fnderr)
	  fprintf (stderr,
		   "(%d) Error for type MPC_INT and op MPC_MIN\n",
		   world_rank);
	free (in);
	free (out);
	free (sol);
      }


      {
	long *in, *out, *sol;
	int i, fnderr = 0;
	in = (long *) malloc (count * sizeof (long));
	out = (long *) malloc (count * sizeof (long));
	sol = (long *) malloc (count * sizeof (long));
	for (i = 0; i < count; i++)
	  {
	    *(in + i) = (rank + i);
	    *(sol + i) = i;
	    *(out + i) = 0;
	  }
	MPC_Allreduce (in, out, count, MPC_LONG, MPC_MIN, comm);
	for (i = 0; i < count; i++)
	  {
	    if (*(out + i) != *(sol + i))
	      {
		errcnt++;
		fnderr++;
	      }
	  }
	if (fnderr)
	  fprintf (stderr,
		   "(%d) Error for type MPC_LONG and op MPC_MIN\n",
		   world_rank);
	free (in);
	free (out);
	free (sol);
      }


      {
	short *in, *out, *sol;
	int i, fnderr = 0;
	in = (short *) malloc (count * sizeof (short));
	out = (short *) malloc (count * sizeof (short));
	sol = (short *) malloc (count * sizeof (short));
	for (i = 0; i < count; i++)
	  {
	    *(in + i) = (rank + i);
	    *(sol + i) = i;
	    *(out + i) = 0;
	  }
	MPC_Allreduce (in, out, count, MPC_SHORT, MPC_MIN, comm);
	for (i = 0; i < count; i++)
	  {
	    if (*(out + i) != *(sol + i))
	      {
		errcnt++;
		fnderr++;
	      }
	  }
	if (fnderr)
	  fprintf (stderr,
		   "(%d) Error for type MPC_SHORT and op MPC_MIN\n",
		   world_rank);
	free (in);
	free (out);
	free (sol);
      }


      {
	unsigned short *in, *out, *sol;
	int i, fnderr = 0;
	in = (unsigned short *) malloc (count * sizeof (unsigned short));
	out = (unsigned short *) malloc (count * sizeof (unsigned short));
	sol = (unsigned short *) malloc (count * sizeof (unsigned short));
	for (i = 0; i < count; i++)
	  {
	    *(in + i) = (rank + i);
	    *(sol + i) = i;
	    *(out + i) = 0;
	  }
	MPC_Allreduce (in, out, count, MPC_UNSIGNED_SHORT, MPC_MIN, comm);
	for (i = 0; i < count; i++)
	  {
	    if (*(out + i) != *(sol + i))
	      {
		errcnt++;
		fnderr++;
	      }
	  }
	if (fnderr)
	  fprintf (stderr,
		   "(%d) Error for type MPC_UNSIGNED_SHORT and op MPC_MIN\n",
		   world_rank);
	free (in);
	free (out);
	free (sol);
      }


      {
	unsigned *in, *out, *sol;
	int i, fnderr = 0;
	in = (unsigned *) malloc (count * sizeof (unsigned));
	out = (unsigned *) malloc (count * sizeof (unsigned));
	sol = (unsigned *) malloc (count * sizeof (unsigned));
	for (i = 0; i < count; i++)
	  {
	    *(in + i) = (rank + i);
	    *(sol + i) = i;
	    *(out + i) = 0;
	  }
	MPC_Allreduce (in, out, count, MPC_UNSIGNED, MPC_MIN, comm);
	for (i = 0; i < count; i++)
	  {
	    if (*(out + i) != *(sol + i))
	      {
		errcnt++;
		fnderr++;
	      }
	  }
	if (fnderr)
	  fprintf (stderr,
		   "(%d) Error for type MPC_UNSIGNED and op MPC_MIN\n",
		   world_rank);
	free (in);
	free (out);
	free (sol);
      }


      {
	unsigned long *in, *out, *sol;
	int i, fnderr = 0;
	in = (unsigned long *) malloc (count * sizeof (unsigned long));
	out = (unsigned long *) malloc (count * sizeof (unsigned long));
	sol = (unsigned long *) malloc (count * sizeof (unsigned long));
	for (i = 0; i < count; i++)
	  {
	    *(in + i) = (rank + i);
	    *(sol + i) = i;
	    *(out + i) = 0;
	  }
	MPC_Allreduce (in, out, count, MPC_UNSIGNED_LONG, MPC_MIN, comm);
	for (i = 0; i < count; i++)
	  {
	    if (*(out + i) != *(sol + i))
	      {
		errcnt++;
		fnderr++;
	      }
	  }
	if (fnderr)
	  fprintf (stderr,
		   "(%d) Error for type MPC_UNSIGNED_LONG and op MPC_MIN\n",
		   world_rank);
	free (in);
	free (out);
	free (sol);
      }


      {
	float *in, *out, *sol;
	int i, fnderr = 0;
	in = (float *) malloc (count * sizeof (float));
	out = (float *) malloc (count * sizeof (float));
	sol = (float *) malloc (count * sizeof (float));
	for (i = 0; i < count; i++)
	  {
	    *(in + i) = (rank + i);
	    *(sol + i) = i;
	    *(out + i) = 0;
	  }
	MPC_Allreduce (in, out, count, MPC_FLOAT, MPC_MIN, comm);
	for (i = 0; i < count; i++)
	  {
	    if (*(out + i) != *(sol + i))
	      {
		errcnt++;
		fnderr++;
	      }
	  }
	if (fnderr)
	  fprintf (stderr,
		   "(%d) Error for type MPC_FLOAT and op MPC_MIN\n",
		   world_rank);
	free (in);
	free (out);
	free (sol);
      }


      {
	double *in, *out, *sol;
	int i, fnderr = 0;
	in = (double *) malloc (count * sizeof (double));
	out = (double *) malloc (count * sizeof (double));
	sol = (double *) malloc (count * sizeof (double));
	for (i = 0; i < count; i++)
	  {
	    *(in + i) = (rank + i);
	    *(sol + i) = i;
	    *(out + i) = 0;
	  }
	MPC_Allreduce (in, out, count, MPC_DOUBLE, MPC_MIN, comm);
	for (i = 0; i < count; i++)
	  {
	    if (*(out + i) != *(sol + i))
	      {
		errcnt++;
		fnderr++;
	      }
	  }
	if (fnderr)
	  fprintf (stderr,
		   "(%d) Error for type MPC_DOUBLE and op MPC_MIN\n",
		   world_rank);
	free (in);
	free (out);
	free (sol);
      }


      gerr += errcnt;
      if (errcnt > 0)
	printf ("Found %d errors on %d for MPC_MIN\n", errcnt, rank);
      errcnt = 0;

/* Test LOR */
      if (world_rank == 0)
	printf ("Testing MPC_LOR...\n");

      {
	int *in, *out, *sol;
	int i, fnderr = 0;
	in = (int *) malloc (count * sizeof (int));
	out = (int *) malloc (count * sizeof (int));
	sol = (int *) malloc (count * sizeof (int));
	for (i = 0; i < count; i++)
	  {
	    *(in + i) = (rank & 0x1);
	    *(sol + i) = (size > 1);
	    *(out + i) = 0;
	  }
	MPC_Allreduce (in, out, count, MPC_INT, MPC_LOR, comm);
	for (i = 0; i < count; i++)
	  {
	    if (*(out + i) != *(sol + i))
	      {
		errcnt++;
		fnderr++;
	      }
	  }
	if (fnderr)
	  fprintf (stderr,
		   "(%d) Error for type MPC_INT and op MPC_LOR\n",
		   world_rank);
	free (in);
	free (out);
	free (sol);
      }


      {
	long *in, *out, *sol;
	int i, fnderr = 0;
	in = (long *) malloc (count * sizeof (long));
	out = (long *) malloc (count * sizeof (long));
	sol = (long *) malloc (count * sizeof (long));
	for (i = 0; i < count; i++)
	  {
	    *(in + i) = (rank & 0x1);
	    *(sol + i) = (size > 1);
	    *(out + i) = 0;
	  }
	MPC_Allreduce (in, out, count, MPC_LONG, MPC_LOR, comm);
	for (i = 0; i < count; i++)
	  {
	    if (*(out + i) != *(sol + i))
	      {
		errcnt++;
		fnderr++;
	      }
	  }
	if (fnderr)
	  fprintf (stderr,
		   "(%d) Error for type MPC_LONG and op MPC_LOR\n",
		   world_rank);
	free (in);
	free (out);
	free (sol);
      }


      {
	short *in, *out, *sol;
	int i, fnderr = 0;
	in = (short *) malloc (count * sizeof (short));
	out = (short *) malloc (count * sizeof (short));
	sol = (short *) malloc (count * sizeof (short));
	for (i = 0; i < count; i++)
	  {
	    *(in + i) = (rank & 0x1);
	    *(sol + i) = (size > 1);
	    *(out + i) = 0;
	  }
	MPC_Allreduce (in, out, count, MPC_SHORT, MPC_LOR, comm);
	for (i = 0; i < count; i++)
	  {
	    if (*(out + i) != *(sol + i))
	      {
		errcnt++;
		fnderr++;
	      }
	  }
	if (fnderr)
	  fprintf (stderr,
		   "(%d) Error for type MPC_SHORT and op MPC_LOR\n",
		   world_rank);
	free (in);
	free (out);
	free (sol);
      }


      {
	unsigned short *in, *out, *sol;
	int i, fnderr = 0;
	in = (unsigned short *) malloc (count * sizeof (unsigned short));
	out = (unsigned short *) malloc (count * sizeof (unsigned short));
	sol = (unsigned short *) malloc (count * sizeof (unsigned short));
	for (i = 0; i < count; i++)
	  {
	    *(in + i) = (rank & 0x1);
	    *(sol + i) = (size > 1);
	    *(out + i) = 0;
	  }
	MPC_Allreduce (in, out, count, MPC_UNSIGNED_SHORT, MPC_LOR, comm);
	for (i = 0; i < count; i++)
	  {
	    if (*(out + i) != *(sol + i))
	      {
		errcnt++;
		fnderr++;
	      }
	  }
	if (fnderr)
	  fprintf (stderr,
		   "(%d) Error for type MPC_UNSIGNED_SHORT and op MPC_LOR\n",
		   world_rank);
	free (in);
	free (out);
	free (sol);
      }


      {
	unsigned *in, *out, *sol;
	int i, fnderr = 0;
	in = (unsigned *) malloc (count * sizeof (unsigned));
	out = (unsigned *) malloc (count * sizeof (unsigned));
	sol = (unsigned *) malloc (count * sizeof (unsigned));
	for (i = 0; i < count; i++)
	  {
	    *(in + i) = (rank & 0x1);
	    *(sol + i) = (size > 1);
	    *(out + i) = 0;
	  }
	MPC_Allreduce (in, out, count, MPC_UNSIGNED, MPC_LOR, comm);
	for (i = 0; i < count; i++)
	  {
	    if (*(out + i) != *(sol + i))
	      {
		errcnt++;
		fnderr++;
	      }
	  }
	if (fnderr)
	  fprintf (stderr,
		   "(%d) Error for type MPC_UNSIGNED and op MPC_LOR\n",
		   world_rank);
	free (in);
	free (out);
	free (sol);
      }


      {
	unsigned long *in, *out, *sol;
	int i, fnderr = 0;
	in = (unsigned long *) malloc (count * sizeof (unsigned long));
	out = (unsigned long *) malloc (count * sizeof (unsigned long));
	sol = (unsigned long *) malloc (count * sizeof (unsigned long));
	for (i = 0; i < count; i++)
	  {
	    *(in + i) = (rank & 0x1);
	    *(sol + i) = (size > 1);
	    *(out + i) = 0;
	  }
	MPC_Allreduce (in, out, count, MPC_UNSIGNED_LONG, MPC_LOR, comm);
	for (i = 0; i < count; i++)
	  {
	    if (*(out + i) != *(sol + i))
	      {
		errcnt++;
		fnderr++;
	      }
	  }
	if (fnderr)
	  fprintf (stderr,
		   "(%d) Error for type MPC_UNSIGNED_LONG and op MPC_LOR\n",
		   world_rank);
	free (in);
	free (out);
	free (sol);
      }


      gerr += errcnt;
      if (errcnt > 0)
	printf ("Found %d errors on %d for MPC_LOR(1)\n", errcnt, rank);
      errcnt = 0;


      {
	int *in, *out, *sol;
	int i, fnderr = 0;
	in = (int *) malloc (count * sizeof (int));
	out = (int *) malloc (count * sizeof (int));
	sol = (int *) malloc (count * sizeof (int));
	for (i = 0; i < count; i++)
	  {
	    *(in + i) = 0;
	    *(sol + i) = 0;
	    *(out + i) = 0;
	  }
	MPC_Allreduce (in, out, count, MPC_INT, MPC_LOR, comm);
	for (i = 0; i < count; i++)
	  {
	    if (*(out + i) != *(sol + i))
	      {
		errcnt++;
		fnderr++;
	      }
	  }
	if (fnderr)
	  fprintf (stderr,
		   "(%d) Error for type MPC_INT and op MPC_LOR\n",
		   world_rank);
	free (in);
	free (out);
	free (sol);
      }


      {
	long *in, *out, *sol;
	int i, fnderr = 0;
	in = (long *) malloc (count * sizeof (long));
	out = (long *) malloc (count * sizeof (long));
	sol = (long *) malloc (count * sizeof (long));
	for (i = 0; i < count; i++)
	  {
	    *(in + i) = 0;
	    *(sol + i) = 0;
	    *(out + i) = 0;
	  }
	MPC_Allreduce (in, out, count, MPC_LONG, MPC_LOR, comm);
	for (i = 0; i < count; i++)
	  {
	    if (*(out + i) != *(sol + i))
	      {
		errcnt++;
		fnderr++;
	      }
	  }
	if (fnderr)
	  fprintf (stderr,
		   "(%d) Error for type MPC_LONG and op MPC_LOR\n",
		   world_rank);
	free (in);
	free (out);
	free (sol);
      }


      {
	short *in, *out, *sol;
	int i, fnderr = 0;
	in = (short *) malloc (count * sizeof (short));
	out = (short *) malloc (count * sizeof (short));
	sol = (short *) malloc (count * sizeof (short));
	for (i = 0; i < count; i++)
	  {
	    *(in + i) = 0;
	    *(sol + i) = 0;
	    *(out + i) = 0;
	  }
	MPC_Allreduce (in, out, count, MPC_SHORT, MPC_LOR, comm);
	for (i = 0; i < count; i++)
	  {
	    if (*(out + i) != *(sol + i))
	      {
		errcnt++;
		fnderr++;
	      }
	  }
	if (fnderr)
	  fprintf (stderr,
		   "(%d) Error for type MPC_SHORT and op MPC_LOR\n",
		   world_rank);
	free (in);
	free (out);
	free (sol);
      }


      {
	unsigned short *in, *out, *sol;
	int i, fnderr = 0;
	in = (unsigned short *) malloc (count * sizeof (unsigned short));
	out = (unsigned short *) malloc (count * sizeof (unsigned short));
	sol = (unsigned short *) malloc (count * sizeof (unsigned short));
	for (i = 0; i < count; i++)
	  {
	    *(in + i) = 0;
	    *(sol + i) = 0;
	    *(out + i) = 0;
	  }
	MPC_Allreduce (in, out, count, MPC_UNSIGNED_SHORT, MPC_LOR, comm);
	for (i = 0; i < count; i++)
	  {
	    if (*(out + i) != *(sol + i))
	      {
		errcnt++;
		fnderr++;
	      }
	  }
	if (fnderr)
	  fprintf (stderr,
		   "(%d) Error for type MPC_UNSIGNED_SHORT and op MPC_LOR\n",
		   world_rank);
	free (in);
	free (out);
	free (sol);
      }


      {
	unsigned *in, *out, *sol;
	int i, fnderr = 0;
	in = (unsigned *) malloc (count * sizeof (unsigned));
	out = (unsigned *) malloc (count * sizeof (unsigned));
	sol = (unsigned *) malloc (count * sizeof (unsigned));
	for (i = 0; i < count; i++)
	  {
	    *(in + i) = 0;
	    *(sol + i) = 0;
	    *(out + i) = 0;
	  }
	MPC_Allreduce (in, out, count, MPC_UNSIGNED, MPC_LOR, comm);
	for (i = 0; i < count; i++)
	  {
	    if (*(out + i) != *(sol + i))
	      {
		errcnt++;
		fnderr++;
	      }
	  }
	if (fnderr)
	  fprintf (stderr,
		   "(%d) Error for type MPC_UNSIGNED and op MPC_LOR\n",
		   world_rank);
	free (in);
	free (out);
	free (sol);
      }


      {
	unsigned long *in, *out, *sol;
	int i, fnderr = 0;
	in = (unsigned long *) malloc (count * sizeof (unsigned long));
	out = (unsigned long *) malloc (count * sizeof (unsigned long));
	sol = (unsigned long *) malloc (count * sizeof (unsigned long));
	for (i = 0; i < count; i++)
	  {
	    *(in + i) = 0;
	    *(sol + i) = 0;
	    *(out + i) = 0;
	  }
	MPC_Allreduce (in, out, count, MPC_UNSIGNED_LONG, MPC_LOR, comm);
	for (i = 0; i < count; i++)
	  {
	    if (*(out + i) != *(sol + i))
	      {
		errcnt++;
		fnderr++;
	      }
	  }
	if (fnderr)
	  fprintf (stderr,
		   "(%d) Error for type MPC_UNSIGNED_LONG and op MPC_LOR\n",
		   world_rank);
	free (in);
	free (out);
	free (sol);
      }


      gerr += errcnt;
      if (errcnt > 0)
	printf ("Found %d errors on %d for MPC_LOR(0)\n", errcnt, rank);
      errcnt = 0;

/* Test LXOR */
      if (world_rank == 0)
	printf ("Testing MPC_LXOR...\n");

      {
	int *in, *out, *sol;
	int i, fnderr = 0;
	in = (int *) malloc (count * sizeof (int));
	out = (int *) malloc (count * sizeof (int));
	sol = (int *) malloc (count * sizeof (int));
	for (i = 0; i < count; i++)
	  {
	    *(in + i) = (rank == 1);
	    *(sol + i) = (size > 1);
	    *(out + i) = 0;
	  }
	MPC_Allreduce (in, out, count, MPC_INT, MPC_LXOR, comm);
	for (i = 0; i < count; i++)
	  {
	    if (*(out + i) != *(sol + i))
	      {
		errcnt++;
		fnderr++;
	      }
	  }
	if (fnderr)
	  fprintf (stderr,
		   "(%d) Error for type MPC_INT and op MPC_LXOR\n",
		   world_rank);
	free (in);
	free (out);
	free (sol);
      }


      {
	long *in, *out, *sol;
	int i, fnderr = 0;
	in = (long *) malloc (count * sizeof (long));
	out = (long *) malloc (count * sizeof (long));
	sol = (long *) malloc (count * sizeof (long));
	for (i = 0; i < count; i++)
	  {
	    *(in + i) = (rank == 1);
	    *(sol + i) = (size > 1);
	    *(out + i) = 0;
	  }
	MPC_Allreduce (in, out, count, MPC_LONG, MPC_LXOR, comm);
	for (i = 0; i < count; i++)
	  {
	    if (*(out + i) != *(sol + i))
	      {
		errcnt++;
		fnderr++;
	      }
	  }
	if (fnderr)
	  fprintf (stderr,
		   "(%d) Error for type MPC_LONG and op MPC_LXOR\n",
		   world_rank);
	free (in);
	free (out);
	free (sol);
      }


      {
	short *in, *out, *sol;
	int i, fnderr = 0;
	in = (short *) malloc (count * sizeof (short));
	out = (short *) malloc (count * sizeof (short));
	sol = (short *) malloc (count * sizeof (short));
	for (i = 0; i < count; i++)
	  {
	    *(in + i) = (rank == 1);
	    *(sol + i) = (size > 1);
	    *(out + i) = 0;
	  }
	MPC_Allreduce (in, out, count, MPC_SHORT, MPC_LXOR, comm);
	for (i = 0; i < count; i++)
	  {
	    if (*(out + i) != *(sol + i))
	      {
		errcnt++;
		fnderr++;
	      }
	  }
	if (fnderr)
	  fprintf (stderr,
		   "(%d) Error for type MPC_SHORT and op MPC_LXOR\n",
		   world_rank);
	free (in);
	free (out);
	free (sol);
      }


      {
	unsigned short *in, *out, *sol;
	int i, fnderr = 0;
	in = (unsigned short *) malloc (count * sizeof (unsigned short));
	out = (unsigned short *) malloc (count * sizeof (unsigned short));
	sol = (unsigned short *) malloc (count * sizeof (unsigned short));
	for (i = 0; i < count; i++)
	  {
	    *(in + i) = (rank == 1);
	    *(sol + i) = (size > 1);
	    *(out + i) = 0;
	  }
	MPC_Allreduce (in, out, count, MPC_UNSIGNED_SHORT, MPC_LXOR, comm);
	for (i = 0; i < count; i++)
	  {
	    if (*(out + i) != *(sol + i))
	      {
		errcnt++;
		fnderr++;
	      }
	  }
	if (fnderr)
	  fprintf (stderr,
		   "(%d) Error for type MPC_UNSIGNED_SHORT and op MPC_LXOR\n",
		   world_rank);
	free (in);
	free (out);
	free (sol);
      }


      {
	unsigned *in, *out, *sol;
	int i, fnderr = 0;
	in = (unsigned *) malloc (count * sizeof (unsigned));
	out = (unsigned *) malloc (count * sizeof (unsigned));
	sol = (unsigned *) malloc (count * sizeof (unsigned));
	for (i = 0; i < count; i++)
	  {
	    *(in + i) = (rank == 1);
	    *(sol + i) = (size > 1);
	    *(out + i) = 0;
	  }
	MPC_Allreduce (in, out, count, MPC_UNSIGNED, MPC_LXOR, comm);
	for (i = 0; i < count; i++)
	  {
	    if (*(out + i) != *(sol + i))
	      {
		errcnt++;
		fnderr++;
	      }
	  }
	if (fnderr)
	  fprintf (stderr,
		   "(%d) Error for type MPC_UNSIGNED and op MPC_LXOR\n",
		   world_rank);
	free (in);
	free (out);
	free (sol);
      }


      {
	unsigned long *in, *out, *sol;
	int i, fnderr = 0;
	in = (unsigned long *) malloc (count * sizeof (unsigned long));
	out = (unsigned long *) malloc (count * sizeof (unsigned long));
	sol = (unsigned long *) malloc (count * sizeof (unsigned long));
	for (i = 0; i < count; i++)
	  {
	    *(in + i) = (rank == 1);
	    *(sol + i) = (size > 1);
	    *(out + i) = 0;
	  }
	MPC_Allreduce (in, out, count, MPC_UNSIGNED_LONG, MPC_LXOR, comm);
	for (i = 0; i < count; i++)
	  {
	    if (*(out + i) != *(sol + i))
	      {
		errcnt++;
		fnderr++;
	      }
	  }
	if (fnderr)
	  fprintf (stderr,
		   "(%d) Error for type MPC_UNSIGNED_LONG and op MPC_LXOR\n",
		   world_rank);
	free (in);
	free (out);
	free (sol);
      }


      gerr += errcnt;
      if (errcnt > 0)
	printf ("Found %d errors on %d for MPC_LXOR(1)\n", errcnt, rank);
      errcnt = 0;


      {
	int *in, *out, *sol;
	int i, fnderr = 0;
	in = (int *) malloc (count * sizeof (int));
	out = (int *) malloc (count * sizeof (int));
	sol = (int *) malloc (count * sizeof (int));
	for (i = 0; i < count; i++)
	  {
	    *(in + i) = 0;
	    *(sol + i) = 0;
	    *(out + i) = 0;
	  }
	MPC_Allreduce (in, out, count, MPC_INT, MPC_LXOR, comm);
	for (i = 0; i < count; i++)
	  {
	    if (*(out + i) != *(sol + i))
	      {
		errcnt++;
		fnderr++;
	      }
	  }
	if (fnderr)
	  fprintf (stderr,
		   "(%d) Error for type MPC_INT and op MPC_LXOR\n",
		   world_rank);
	free (in);
	free (out);
	free (sol);
      }


      {
	long *in, *out, *sol;
	int i, fnderr = 0;
	in = (long *) malloc (count * sizeof (long));
	out = (long *) malloc (count * sizeof (long));
	sol = (long *) malloc (count * sizeof (long));
	for (i = 0; i < count; i++)
	  {
	    *(in + i) = 0;
	    *(sol + i) = 0;
	    *(out + i) = 0;
	  }
	MPC_Allreduce (in, out, count, MPC_LONG, MPC_LXOR, comm);
	for (i = 0; i < count; i++)
	  {
	    if (*(out + i) != *(sol + i))
	      {
		errcnt++;
		fnderr++;
	      }
	  }
	if (fnderr)
	  fprintf (stderr,
		   "(%d) Error for type MPC_LONG and op MPC_LXOR\n",
		   world_rank);
	free (in);
	free (out);
	free (sol);
      }


      {
	short *in, *out, *sol;
	int i, fnderr = 0;
	in = (short *) malloc (count * sizeof (short));
	out = (short *) malloc (count * sizeof (short));
	sol = (short *) malloc (count * sizeof (short));
	for (i = 0; i < count; i++)
	  {
	    *(in + i) = 0;
	    *(sol + i) = 0;
	    *(out + i) = 0;
	  }
	MPC_Allreduce (in, out, count, MPC_SHORT, MPC_LXOR, comm);
	for (i = 0; i < count; i++)
	  {
	    if (*(out + i) != *(sol + i))
	      {
		errcnt++;
		fnderr++;
	      }
	  }
	if (fnderr)
	  fprintf (stderr,
		   "(%d) Error for type MPC_SHORT and op MPC_LXOR\n",
		   world_rank);
	free (in);
	free (out);
	free (sol);
      }


      {
	unsigned short *in, *out, *sol;
	int i, fnderr = 0;
	in = (unsigned short *) malloc (count * sizeof (unsigned short));
	out = (unsigned short *) malloc (count * sizeof (unsigned short));
	sol = (unsigned short *) malloc (count * sizeof (unsigned short));
	for (i = 0; i < count; i++)
	  {
	    *(in + i) = 0;
	    *(sol + i) = 0;
	    *(out + i) = 0;
	  }
	MPC_Allreduce (in, out, count, MPC_UNSIGNED_SHORT, MPC_LXOR, comm);
	for (i = 0; i < count; i++)
	  {
	    if (*(out + i) != *(sol + i))
	      {
		errcnt++;
		fnderr++;
	      }
	  }
	if (fnderr)
	  fprintf (stderr,
		   "(%d) Error for type MPC_UNSIGNED_SHORT and op MPC_LXOR\n",
		   world_rank);
	free (in);
	free (out);
	free (sol);
      }


      {
	unsigned *in, *out, *sol;
	int i, fnderr = 0;
	in = (unsigned *) malloc (count * sizeof (unsigned));
	out = (unsigned *) malloc (count * sizeof (unsigned));
	sol = (unsigned *) malloc (count * sizeof (unsigned));
	for (i = 0; i < count; i++)
	  {
	    *(in + i) = 0;
	    *(sol + i) = 0;
	    *(out + i) = 0;
	  }
	MPC_Allreduce (in, out, count, MPC_UNSIGNED, MPC_LXOR, comm);
	for (i = 0; i < count; i++)
	  {
	    if (*(out + i) != *(sol + i))
	      {
		errcnt++;
		fnderr++;
	      }
	  }
	if (fnderr)
	  fprintf (stderr,
		   "(%d) Error for type MPC_UNSIGNED and op MPC_LXOR\n",
		   world_rank);
	free (in);
	free (out);
	free (sol);
      }


      {
	unsigned long *in, *out, *sol;
	int i, fnderr = 0;
	in = (unsigned long *) malloc (count * sizeof (unsigned long));
	out = (unsigned long *) malloc (count * sizeof (unsigned long));
	sol = (unsigned long *) malloc (count * sizeof (unsigned long));
	for (i = 0; i < count; i++)
	  {
	    *(in + i) = 0;
	    *(sol + i) = 0;
	    *(out + i) = 0;
	  }
	MPC_Allreduce (in, out, count, MPC_UNSIGNED_LONG, MPC_LXOR, comm);
	for (i = 0; i < count; i++)
	  {
	    if (*(out + i) != *(sol + i))
	      {
		errcnt++;
		fnderr++;
	      }
	  }
	if (fnderr)
	  fprintf (stderr,
		   "(%d) Error for type MPC_UNSIGNED_LONG and op MPC_LXOR\n",
		   world_rank);
	free (in);
	free (out);
	free (sol);
      }


      gerr += errcnt;
      if (errcnt > 0)
	printf ("Found %d errors on %d for MPC_LXOR(0)\n", errcnt, rank);
      errcnt = 0;


      {
	int *in, *out, *sol;
	int i, fnderr = 0;
	in = (int *) malloc (count * sizeof (int));
	out = (int *) malloc (count * sizeof (int));
	sol = (int *) malloc (count * sizeof (int));
	for (i = 0; i < count; i++)
	  {
	    *(in + i) = 1;
	    *(sol + i) = 0;
	    *(out + i) = 0;
	  }
	MPC_Allreduce (in, out, count, MPC_INT, MPC_LXOR, comm);
	for (i = 0; i < count; i++)
	  {
	    if (*(out + i) != *(sol + i))
	      {
		errcnt++;
		fnderr++;
	      }
	  }
	if (fnderr)
	  fprintf (stderr,
		   "(%d) Error for type MPC_INT and op MPC_LXOR\n",
		   world_rank);
	free (in);
	free (out);
	free (sol);
      }


      {
	long *in, *out, *sol;
	int i, fnderr = 0;
	in = (long *) malloc (count * sizeof (long));
	out = (long *) malloc (count * sizeof (long));
	sol = (long *) malloc (count * sizeof (long));
	for (i = 0; i < count; i++)
	  {
	    *(in + i) = 1;
	    *(sol + i) = 0;
	    *(out + i) = 0;
	  }
	MPC_Allreduce (in, out, count, MPC_LONG, MPC_LXOR, comm);
	for (i = 0; i < count; i++)
	  {
	    if (*(out + i) != *(sol + i))
	      {
		errcnt++;
		fnderr++;
	      }
	  }
	if (fnderr)
	  fprintf (stderr,
		   "(%d) Error for type MPC_LONG and op MPC_LXOR\n",
		   world_rank);
	free (in);
	free (out);
	free (sol);
      }


      {
	short *in, *out, *sol;
	int i, fnderr = 0;
	in = (short *) malloc (count * sizeof (short));
	out = (short *) malloc (count * sizeof (short));
	sol = (short *) malloc (count * sizeof (short));
	for (i = 0; i < count; i++)
	  {
	    *(in + i) = 1;
	    *(sol + i) = 0;
	    *(out + i) = 0;
	  }
	MPC_Allreduce (in, out, count, MPC_SHORT, MPC_LXOR, comm);
	for (i = 0; i < count; i++)
	  {
	    if (*(out + i) != *(sol + i))
	      {
		errcnt++;
		fnderr++;
	      }
	  }
	if (fnderr)
	  fprintf (stderr,
		   "(%d) Error for type MPC_SHORT and op MPC_LXOR\n",
		   world_rank);
	free (in);
	free (out);
	free (sol);
      }


      {
	unsigned short *in, *out, *sol;
	int i, fnderr = 0;
	in = (unsigned short *) malloc (count * sizeof (unsigned short));
	out = (unsigned short *) malloc (count * sizeof (unsigned short));
	sol = (unsigned short *) malloc (count * sizeof (unsigned short));
	for (i = 0; i < count; i++)
	  {
	    *(in + i) = 1;
	    *(sol + i) = 0;
	    *(out + i) = 0;
	  }
	MPC_Allreduce (in, out, count, MPC_UNSIGNED_SHORT, MPC_LXOR, comm);
	for (i = 0; i < count; i++)
	  {
	    if (*(out + i) != *(sol + i))
	      {
		errcnt++;
		fnderr++;
	      }
	  }
	if (fnderr)
	  fprintf (stderr,
		   "(%d) Error for type MPC_UNSIGNED_SHORT and op MPC_LXOR\n",
		   world_rank);
	free (in);
	free (out);
	free (sol);
      }


      {
	unsigned *in, *out, *sol;
	int i, fnderr = 0;
	in = (unsigned *) malloc (count * sizeof (unsigned));
	out = (unsigned *) malloc (count * sizeof (unsigned));
	sol = (unsigned *) malloc (count * sizeof (unsigned));
	for (i = 0; i < count; i++)
	  {
	    *(in + i) = 1;
	    *(sol + i) = 0;
	    *(out + i) = 0;
	  }
	MPC_Allreduce (in, out, count, MPC_UNSIGNED, MPC_LXOR, comm);
	for (i = 0; i < count; i++)
	  {
	    if (*(out + i) != *(sol + i))
	      {
		errcnt++;
		fnderr++;
	      }
	  }
	if (fnderr)
	  fprintf (stderr,
		   "(%d) Error for type MPC_UNSIGNED and op MPC_LXOR\n",
		   world_rank);
	free (in);
	free (out);
	free (sol);
      }


      {
	unsigned long *in, *out, *sol;
	int i, fnderr = 0;
	in = (unsigned long *) malloc (count * sizeof (unsigned long));
	out = (unsigned long *) malloc (count * sizeof (unsigned long));
	sol = (unsigned long *) malloc (count * sizeof (unsigned long));
	for (i = 0; i < count; i++)
	  {
	    *(in + i) = 1;
	    *(sol + i) = 0;
	    *(out + i) = 0;
	  }
	MPC_Allreduce (in, out, count, MPC_UNSIGNED_LONG, MPC_LXOR, comm);
	for (i = 0; i < count; i++)
	  {
	    if (*(out + i) != *(sol + i))
	      {
		errcnt++;
		fnderr++;
	      }
	  }
	if (fnderr)
	  fprintf (stderr,
		   "(%d) Error for type MPC_UNSIGNED_LONG and op MPC_LXOR\n",
		   world_rank);
	free (in);
	free (out);
	free (sol);
      }


      gerr += errcnt;
      if (errcnt > 0)
	printf ("Found %d errors on %d for MPC_LXOR(1-0)\n", errcnt, rank);
      errcnt = 0;

/* Test LAND */
      if (world_rank == 0)
	printf ("Testing MPC_LAND...\n");

      {
	int *in, *out, *sol;
	int i, fnderr = 0;
	in = (int *) malloc (count * sizeof (int));
	out = (int *) malloc (count * sizeof (int));
	sol = (int *) malloc (count * sizeof (int));
	for (i = 0; i < count; i++)
	  {
	    *(in + i) = (rank & 0x1);
	    *(sol + i) = 0;
	    *(out + i) = 0;
	  }
	MPC_Allreduce (in, out, count, MPC_INT, MPC_LAND, comm);
	for (i = 0; i < count; i++)
	  {
	    if (*(out + i) != *(sol + i))
	      {
		errcnt++;
		fnderr++;
	      }
	  }
	if (fnderr)
	  fprintf (stderr,
		   "(%d) Error for type MPC_INT and op MPC_LAND\n",
		   world_rank);
	free (in);
	free (out);
	free (sol);
      }


      {
	long *in, *out, *sol;
	int i, fnderr = 0;
	in = (long *) malloc (count * sizeof (long));
	out = (long *) malloc (count * sizeof (long));
	sol = (long *) malloc (count * sizeof (long));
	for (i = 0; i < count; i++)
	  {
	    *(in + i) = (rank & 0x1);
	    *(sol + i) = 0;
	    *(out + i) = 0;
	  }
	MPC_Allreduce (in, out, count, MPC_LONG, MPC_LAND, comm);
	for (i = 0; i < count; i++)
	  {
	    if (*(out + i) != *(sol + i))
	      {
		errcnt++;
		fnderr++;
	      }
	  }
	if (fnderr)
	  fprintf (stderr,
		   "(%d) Error for type MPC_LONG and op MPC_LAND\n",
		   world_rank);
	free (in);
	free (out);
	free (sol);
      }


      {
	short *in, *out, *sol;
	int i, fnderr = 0;
	in = (short *) malloc (count * sizeof (short));
	out = (short *) malloc (count * sizeof (short));
	sol = (short *) malloc (count * sizeof (short));
	for (i = 0; i < count; i++)
	  {
	    *(in + i) = (rank & 0x1);
	    *(sol + i) = 0;
	    *(out + i) = 0;
	  }
	MPC_Allreduce (in, out, count, MPC_SHORT, MPC_LAND, comm);
	for (i = 0; i < count; i++)
	  {
	    if (*(out + i) != *(sol + i))
	      {
		errcnt++;
		fnderr++;
	      }
	  }
	if (fnderr)
	  fprintf (stderr,
		   "(%d) Error for type MPC_SHORT and op MPC_LAND\n",
		   world_rank);
	free (in);
	free (out);
	free (sol);
      }


      {
	unsigned short *in, *out, *sol;
	int i, fnderr = 0;
	in = (unsigned short *) malloc (count * sizeof (unsigned short));
	out = (unsigned short *) malloc (count * sizeof (unsigned short));
	sol = (unsigned short *) malloc (count * sizeof (unsigned short));
	for (i = 0; i < count; i++)
	  {
	    *(in + i) = (rank & 0x1);
	    *(sol + i) = 0;
	    *(out + i) = 0;
	  }
	MPC_Allreduce (in, out, count, MPC_UNSIGNED_SHORT, MPC_LAND, comm);
	for (i = 0; i < count; i++)
	  {
	    if (*(out + i) != *(sol + i))
	      {
		errcnt++;
		fnderr++;
	      }
	  }
	if (fnderr)
	  fprintf (stderr,
		   "(%d) Error for type MPC_UNSIGNED_SHORT and op MPC_LAND\n",
		   world_rank);
	free (in);
	free (out);
	free (sol);
      }


      {
	unsigned *in, *out, *sol;
	int i, fnderr = 0;
	in = (unsigned *) malloc (count * sizeof (unsigned));
	out = (unsigned *) malloc (count * sizeof (unsigned));
	sol = (unsigned *) malloc (count * sizeof (unsigned));
	for (i = 0; i < count; i++)
	  {
	    *(in + i) = (rank & 0x1);
	    *(sol + i) = 0;
	    *(out + i) = 0;
	  }
	MPC_Allreduce (in, out, count, MPC_UNSIGNED, MPC_LAND, comm);
	for (i = 0; i < count; i++)
	  {
	    if (*(out + i) != *(sol + i))
	      {
		errcnt++;
		fnderr++;
	      }
	  }
	if (fnderr)
	  fprintf (stderr,
		   "(%d) Error for type MPC_UNSIGNED and op MPC_LAND\n",
		   world_rank);
	free (in);
	free (out);
	free (sol);
      }


      {
	unsigned long *in, *out, *sol;
	int i, fnderr = 0;
	in = (unsigned long *) malloc (count * sizeof (unsigned long));
	out = (unsigned long *) malloc (count * sizeof (unsigned long));
	sol = (unsigned long *) malloc (count * sizeof (unsigned long));
	for (i = 0; i < count; i++)
	  {
	    *(in + i) = (rank & 0x1);
	    *(sol + i) = 0;
	    *(out + i) = 0;
	  }
	MPC_Allreduce (in, out, count, MPC_UNSIGNED_LONG, MPC_LAND, comm);
	for (i = 0; i < count; i++)
	  {
	    if (*(out + i) != *(sol + i))
	      {
		errcnt++;
		fnderr++;
	      }
	  }
	if (fnderr)
	  fprintf (stderr,
		   "(%d) Error for type MPC_UNSIGNED_LONG and op MPC_LAND\n",
		   world_rank);
	free (in);
	free (out);
	free (sol);
      }


      gerr += errcnt;
      if (errcnt > 0)
	printf ("Found %d errors on %d for MPC_LAND(0)\n", errcnt, rank);
      errcnt = 0;


      {
	int *in, *out, *sol;
	int i, fnderr = 0;
	in = (int *) malloc (count * sizeof (int));
	out = (int *) malloc (count * sizeof (int));
	sol = (int *) malloc (count * sizeof (int));
	for (i = 0; i < count; i++)
	  {
	    *(in + i) = 1;
	    *(sol + i) = 1;
	    *(out + i) = 0;
	  }
	MPC_Allreduce (in, out, count, MPC_INT, MPC_LAND, comm);
	for (i = 0; i < count; i++)
	  {
	    if (*(out + i) != *(sol + i))
	      {
		errcnt++;
		fnderr++;
	      }
	  }
	if (fnderr)
	  fprintf (stderr,
		   "(%d) Error for type MPC_INT and op MPC_LAND\n",
		   world_rank);
	free (in);
	free (out);
	free (sol);
      }


      {
	long *in, *out, *sol;
	int i, fnderr = 0;
	in = (long *) malloc (count * sizeof (long));
	out = (long *) malloc (count * sizeof (long));
	sol = (long *) malloc (count * sizeof (long));
	for (i = 0; i < count; i++)
	  {
	    *(in + i) = 1;
	    *(sol + i) = 1;
	    *(out + i) = 0;
	  }
	MPC_Allreduce (in, out, count, MPC_LONG, MPC_LAND, comm);
	for (i = 0; i < count; i++)
	  {
	    if (*(out + i) != *(sol + i))
	      {
		errcnt++;
		fnderr++;
	      }
	  }
	if (fnderr)
	  fprintf (stderr,
		   "(%d) Error for type MPC_LONG and op MPC_LAND\n",
		   world_rank);
	free (in);
	free (out);
	free (sol);
      }


      {
	short *in, *out, *sol;
	int i, fnderr = 0;
	in = (short *) malloc (count * sizeof (short));
	out = (short *) malloc (count * sizeof (short));
	sol = (short *) malloc (count * sizeof (short));
	for (i = 0; i < count; i++)
	  {
	    *(in + i) = 1;
	    *(sol + i) = 1;
	    *(out + i) = 0;
	  }
	MPC_Allreduce (in, out, count, MPC_SHORT, MPC_LAND, comm);
	for (i = 0; i < count; i++)
	  {
	    if (*(out + i) != *(sol + i))
	      {
		errcnt++;
		fnderr++;
	      }
	  }
	if (fnderr)
	  fprintf (stderr,
		   "(%d) Error for type MPC_SHORT and op MPC_LAND\n",
		   world_rank);
	free (in);
	free (out);
	free (sol);
      }


      {
	unsigned short *in, *out, *sol;
	int i, fnderr = 0;
	in = (unsigned short *) malloc (count * sizeof (unsigned short));
	out = (unsigned short *) malloc (count * sizeof (unsigned short));
	sol = (unsigned short *) malloc (count * sizeof (unsigned short));
	for (i = 0; i < count; i++)
	  {
	    *(in + i) = 1;
	    *(sol + i) = 1;
	    *(out + i) = 0;
	  }
	MPC_Allreduce (in, out, count, MPC_UNSIGNED_SHORT, MPC_LAND, comm);
	for (i = 0; i < count; i++)
	  {
	    if (*(out + i) != *(sol + i))
	      {
		errcnt++;
		fnderr++;
	      }
	  }
	if (fnderr)
	  fprintf (stderr,
		   "(%d) Error for type MPC_UNSIGNED_SHORT and op MPC_LAND\n",
		   world_rank);
	free (in);
	free (out);
	free (sol);
      }


      {
	unsigned *in, *out, *sol;
	int i, fnderr = 0;
	in = (unsigned *) malloc (count * sizeof (unsigned));
	out = (unsigned *) malloc (count * sizeof (unsigned));
	sol = (unsigned *) malloc (count * sizeof (unsigned));
	for (i = 0; i < count; i++)
	  {
	    *(in + i) = 1;
	    *(sol + i) = 1;
	    *(out + i) = 0;
	  }
	MPC_Allreduce (in, out, count, MPC_UNSIGNED, MPC_LAND, comm);
	for (i = 0; i < count; i++)
	  {
	    if (*(out + i) != *(sol + i))
	      {
		errcnt++;
		fnderr++;
	      }
	  }
	if (fnderr)
	  fprintf (stderr,
		   "(%d) Error for type MPC_UNSIGNED and op MPC_LAND\n",
		   world_rank);
	free (in);
	free (out);
	free (sol);
      }


      {
	unsigned long *in, *out, *sol;
	int i, fnderr = 0;
	in = (unsigned long *) malloc (count * sizeof (unsigned long));
	out = (unsigned long *) malloc (count * sizeof (unsigned long));
	sol = (unsigned long *) malloc (count * sizeof (unsigned long));
	for (i = 0; i < count; i++)
	  {
	    *(in + i) = 1;
	    *(sol + i) = 1;
	    *(out + i) = 0;
	  }
	MPC_Allreduce (in, out, count, MPC_UNSIGNED_LONG, MPC_LAND, comm);
	for (i = 0; i < count; i++)
	  {
	    if (*(out + i) != *(sol + i))
	      {
		errcnt++;
		fnderr++;
	      }
	  }
	if (fnderr)
	  fprintf (stderr,
		   "(%d) Error for type MPC_UNSIGNED_LONG and op MPC_LAND\n",
		   world_rank);
	free (in);
	free (out);
	free (sol);
      }


      gerr += errcnt;
      if (errcnt > 0)
	printf ("Found %d errors on %d for MPC_LAND(1)\n", errcnt, rank);
      errcnt = 0;

/* Test BOR */
      if (world_rank == 0)
	printf ("Testing MPC_BOR...\n");

      {
	int *in, *out, *sol;
	int i, fnderr = 0;
	in = (int *) malloc (count * sizeof (int));
	out = (int *) malloc (count * sizeof (int));
	sol = (int *) malloc (count * sizeof (int));
	for (i = 0; i < count; i++)
	  {
	    *(in + i) = rank & 0x3;
	    *(sol + i) = (size < 3) ? size - 1 : 0x3;
	    *(out + i) = 0;
	  }
	MPC_Allreduce (in, out, count, MPC_INT, MPC_BOR, comm);
	for (i = 0; i < count; i++)
	  {
	    if (*(out + i) != *(sol + i))
	      {
		errcnt++;
		fnderr++;
	      }
	  }
	if (fnderr)
	  fprintf (stderr,
		   "(%d) Error for type MPC_INT and op MPC_BOR\n",
		   world_rank);
	free (in);
	free (out);
	free (sol);
      }


      {
	long *in, *out, *sol;
	int i, fnderr = 0;
	in = (long *) malloc (count * sizeof (long));
	out = (long *) malloc (count * sizeof (long));
	sol = (long *) malloc (count * sizeof (long));
	for (i = 0; i < count; i++)
	  {
	    *(in + i) = rank & 0x3;
	    *(sol + i) = (size < 3) ? size - 1 : 0x3;
	    *(out + i) = 0;
	  }
	MPC_Allreduce (in, out, count, MPC_LONG, MPC_BOR, comm);
	for (i = 0; i < count; i++)
	  {
	    if (*(out + i) != *(sol + i))
	      {
		errcnt++;
		fnderr++;
	      }
	  }
	if (fnderr)
	  fprintf (stderr,
		   "(%d) Error for type MPC_LONG and op MPC_BOR\n",
		   world_rank);
	free (in);
	free (out);
	free (sol);
      }


      {
	short *in, *out, *sol;
	int i, fnderr = 0;
	in = (short *) malloc (count * sizeof (short));
	out = (short *) malloc (count * sizeof (short));
	sol = (short *) malloc (count * sizeof (short));
	for (i = 0; i < count; i++)
	  {
	    *(in + i) = rank & 0x3;
	    *(sol + i) = (size < 3) ? size - 1 : 0x3;
	    *(out + i) = 0;
	  }
	MPC_Allreduce (in, out, count, MPC_SHORT, MPC_BOR, comm);
	for (i = 0; i < count; i++)
	  {
	    if (*(out + i) != *(sol + i))
	      {
		errcnt++;
		fnderr++;
	      }
	  }
	if (fnderr)
	  fprintf (stderr,
		   "(%d) Error for type MPC_SHORT and op MPC_BOR\n",
		   world_rank);
	free (in);
	free (out);
	free (sol);
      }


      {
	unsigned short *in, *out, *sol;
	int i, fnderr = 0;
	in = (unsigned short *) malloc (count * sizeof (unsigned short));
	out = (unsigned short *) malloc (count * sizeof (unsigned short));
	sol = (unsigned short *) malloc (count * sizeof (unsigned short));
	for (i = 0; i < count; i++)
	  {
	    *(in + i) = rank & 0x3;
	    *(sol + i) = (size < 3) ? size - 1 : 0x3;
	    *(out + i) = 0;
	  }
	MPC_Allreduce (in, out, count, MPC_UNSIGNED_SHORT, MPC_BOR, comm);
	for (i = 0; i < count; i++)
	  {
	    if (*(out + i) != *(sol + i))
	      {
		errcnt++;
		fnderr++;
	      }
	  }
	if (fnderr)
	  fprintf (stderr,
		   "(%d) Error for type MPC_UNSIGNED_SHORT and op MPC_BOR\n",
		   world_rank);
	free (in);
	free (out);
	free (sol);
      }


      {
	unsigned *in, *out, *sol;
	int i, fnderr = 0;
	in = (unsigned *) malloc (count * sizeof (unsigned));
	out = (unsigned *) malloc (count * sizeof (unsigned));
	sol = (unsigned *) malloc (count * sizeof (unsigned));
	for (i = 0; i < count; i++)
	  {
	    *(in + i) = rank & 0x3;
	    *(sol + i) = (size < 3) ? size - 1 : 0x3;
	    *(out + i) = 0;
	  }
	MPC_Allreduce (in, out, count, MPC_UNSIGNED, MPC_BOR, comm);
	for (i = 0; i < count; i++)
	  {
	    if (*(out + i) != *(sol + i))
	      {
		errcnt++;
		fnderr++;
	      }
	  }
	if (fnderr)
	  fprintf (stderr,
		   "(%d) Error for type MPC_UNSIGNED and op MPC_BOR\n",
		   world_rank);
	free (in);
	free (out);
	free (sol);
      }


      {
	unsigned long *in, *out, *sol;
	int i, fnderr = 0;
	in = (unsigned long *) malloc (count * sizeof (unsigned long));
	out = (unsigned long *) malloc (count * sizeof (unsigned long));
	sol = (unsigned long *) malloc (count * sizeof (unsigned long));
	for (i = 0; i < count; i++)
	  {
	    *(in + i) = rank & 0x3;
	    *(sol + i) = (size < 3) ? size - 1 : 0x3;
	    *(out + i) = 0;
	  }
	MPC_Allreduce (in, out, count, MPC_UNSIGNED_LONG, MPC_BOR, comm);
	for (i = 0; i < count; i++)
	  {
	    if (*(out + i) != *(sol + i))
	      {
		errcnt++;
		fnderr++;
	      }
	  }
	if (fnderr)
	  fprintf (stderr,
		   "(%d) Error for type MPC_UNSIGNED_LONG and op MPC_BOR\n",
		   world_rank);
	free (in);
	free (out);
	free (sol);
      }


      {
	unsigned char *in, *out, *sol;
	int i, fnderr = 0;
	in = (unsigned char *) malloc (count * sizeof (unsigned char));
	out = (unsigned char *) malloc (count * sizeof (unsigned char));
	sol = (unsigned char *) malloc (count * sizeof (unsigned char));
	for (i = 0; i < count; i++)
	  {
	    *(in + i) = rank & 0x3;
	    *(sol + i) = (size < 3) ? size - 1 : 0x3;
	    *(out + i) = 0;
	  }
	MPC_Allreduce (in, out, count, MPC_BYTE, MPC_BOR, comm);
	for (i = 0; i < count; i++)
	  {
	    if (*(out + i) != *(sol + i))
	      {
		errcnt++;
		fnderr++;
	      }
	  }
	if (fnderr)
	  fprintf (stderr,
		   "(%d) Error for type MPC_BYTE and op MPC_BOR\n",
		   world_rank);
	free (in);
	free (out);
	free (sol);
      }


      gerr += errcnt;
      if (errcnt > 0)
	printf ("Found %d errors on %d for MPC_BOR(1)\n", errcnt, rank);
      errcnt = 0;

/* Test BAND */
      if (world_rank == 0)
	printf ("Testing MPC_BAND...\n");

      {
	int *in, *out, *sol;
	int i, fnderr = 0;
	in = (int *) malloc (count * sizeof (int));
	out = (int *) malloc (count * sizeof (int));
	sol = (int *) malloc (count * sizeof (int));
	for (i = 0; i < count; i++)
	  {
	    *(in + i) = (rank == size - 1 ? i : ~0);
	    *(sol + i) = i;
	    *(out + i) = 0;
	  }
	MPC_Allreduce (in, out, count, MPC_INT, MPC_BAND, comm);
	for (i = 0; i < count; i++)
	  {
	    if (*(out + i) != *(sol + i))
	      {
		errcnt++;
		fnderr++;
	      }
	  }
	if (fnderr)
	  fprintf (stderr,
		   "(%d) Error for type MPC_INT and op MPC_BAND\n",
		   world_rank);
	free (in);
	free (out);
	free (sol);
      }


      {
	long *in, *out, *sol;
	int i, fnderr = 0;
	in = (long *) malloc (count * sizeof (long));
	out = (long *) malloc (count * sizeof (long));
	sol = (long *) malloc (count * sizeof (long));
	for (i = 0; i < count; i++)
	  {
	    *(in + i) = (rank == size - 1 ? i : ~0);
	    *(sol + i) = i;
	    *(out + i) = 0;
	  }
	MPC_Allreduce (in, out, count, MPC_LONG, MPC_BAND, comm);
	for (i = 0; i < count; i++)
	  {
	    if (*(out + i) != *(sol + i))
	      {
		errcnt++;
		fnderr++;
	      }
	  }
	if (fnderr)
	  fprintf (stderr,
		   "(%d) Error for type MPC_LONG and op MPC_BAND\n",
		   world_rank);
	free (in);
	free (out);
	free (sol);
      }


      {
	short *in, *out, *sol;
	int i, fnderr = 0;
	in = (short *) malloc (count * sizeof (short));
	out = (short *) malloc (count * sizeof (short));
	sol = (short *) malloc (count * sizeof (short));
	for (i = 0; i < count; i++)
	  {
	    *(in + i) = (rank == size - 1 ? i : ~0);
	    *(sol + i) = i;
	    *(out + i) = 0;
	  }
	MPC_Allreduce (in, out, count, MPC_SHORT, MPC_BAND, comm);
	for (i = 0; i < count; i++)
	  {
	    if (*(out + i) != *(sol + i))
	      {
		errcnt++;
		fnderr++;
	      }
	  }
	if (fnderr)
	  fprintf (stderr,
		   "(%d) Error for type MPC_SHORT and op MPC_BAND\n",
		   world_rank);
	free (in);
	free (out);
	free (sol);
      }


      {
	unsigned short *in, *out, *sol;
	int i, fnderr = 0;
	in = (unsigned short *) malloc (count * sizeof (unsigned short));
	out = (unsigned short *) malloc (count * sizeof (unsigned short));
	sol = (unsigned short *) malloc (count * sizeof (unsigned short));
	for (i = 0; i < count; i++)
	  {
	    *(in + i) = (rank == size - 1 ? i : ~0);
	    *(sol + i) = i;
	    *(out + i) = 0;
	  }
	MPC_Allreduce (in, out, count, MPC_UNSIGNED_SHORT, MPC_BAND, comm);
	for (i = 0; i < count; i++)
	  {
	    if (*(out + i) != *(sol + i))
	      {
		errcnt++;
		fnderr++;
	      }
	  }
	if (fnderr)
	  fprintf (stderr,
		   "(%d) Error for type MPC_UNSIGNED_SHORT and op MPC_BAND\n",
		   world_rank);
	free (in);
	free (out);
	free (sol);
      }


      {
	unsigned *in, *out, *sol;
	int i, fnderr = 0;
	in = (unsigned *) malloc (count * sizeof (unsigned));
	out = (unsigned *) malloc (count * sizeof (unsigned));
	sol = (unsigned *) malloc (count * sizeof (unsigned));
	for (i = 0; i < count; i++)
	  {
	    *(in + i) = (rank == size - 1 ? i : ~0);
	    *(sol + i) = i;
	    *(out + i) = 0;
	  }
	MPC_Allreduce (in, out, count, MPC_UNSIGNED, MPC_BAND, comm);
	for (i = 0; i < count; i++)
	  {
	    if (*(out + i) != *(sol + i))
	      {
		errcnt++;
		fnderr++;
	      }
	  }
	if (fnderr)
	  fprintf (stderr,
		   "(%d) Error for type MPC_UNSIGNED and op MPC_BAND\n",
		   world_rank);
	free (in);
	free (out);
	free (sol);
      }


      {
	unsigned long *in, *out, *sol;
	int i, fnderr = 0;
	in = (unsigned long *) malloc (count * sizeof (unsigned long));
	out = (unsigned long *) malloc (count * sizeof (unsigned long));
	sol = (unsigned long *) malloc (count * sizeof (unsigned long));
	for (i = 0; i < count; i++)
	  {
	    *(in + i) = (rank == size - 1 ? i : ~0);
	    *(sol + i) = i;
	    *(out + i) = 0;
	  }
	MPC_Allreduce (in, out, count, MPC_UNSIGNED_LONG, MPC_BAND, comm);
	for (i = 0; i < count; i++)
	  {
	    if (*(out + i) != *(sol + i))
	      {
		errcnt++;
		fnderr++;
	      }
	  }
	if (fnderr)
	  fprintf (stderr,
		   "(%d) Error for type MPC_UNSIGNED_LONG and op MPC_BAND\n",
		   world_rank);
	free (in);
	free (out);
	free (sol);
      }


      {
	unsigned char *in, *out, *sol;
	int i, fnderr = 0;
	in = (unsigned char *) malloc (count * sizeof (unsigned char));
	out = (unsigned char *) malloc (count * sizeof (unsigned char));
	sol = (unsigned char *) malloc (count * sizeof (unsigned char));
	for (i = 0; i < count; i++)
	  {
	    *(in + i) = (rank == size - 1 ? i : ~0);
	    *(sol + i) = i;
	    *(out + i) = 0;
	  }
	MPC_Allreduce (in, out, count, MPC_BYTE, MPC_BAND, comm);
	for (i = 0; i < count; i++)
	  {
	    if (*(out + i) != *(sol + i))
	      {
		errcnt++;
		fnderr++;
	      }
	  }
	if (fnderr)
	  fprintf (stderr,
		   "(%d) Error for type MPC_BYTE and op MPC_BAND\n",
		   world_rank);
	free (in);
	free (out);
	free (sol);
      }


      gerr += errcnt;
      if (errcnt > 0)
	printf ("Found %d errors on %d for MPC_BAND(1)\n", errcnt, rank);
      errcnt = 0;


      {
	int *in, *out, *sol;
	int i, fnderr = 0;
	in = (int *) malloc (count * sizeof (int));
	out = (int *) malloc (count * sizeof (int));
	sol = (int *) malloc (count * sizeof (int));
	for (i = 0; i < count; i++)
	  {
	    *(in + i) = (rank == size - 1 ? i : 0);
	    *(sol + i) = 0;
	    *(out + i) = 0;
	  }
	MPC_Allreduce (in, out, count, MPC_INT, MPC_BAND, comm);
	for (i = 0; i < count; i++)
	  {
	    if (*(out + i) != *(sol + i))
	      {
		errcnt++;
		fnderr++;
	      }
	  }
	if (fnderr)
	  fprintf (stderr,
		   "(%d) Error for type MPC_INT and op MPC_BAND\n",
		   world_rank);
	free (in);
	free (out);
	free (sol);
      }


      {
	long *in, *out, *sol;
	int i, fnderr = 0;
	in = (long *) malloc (count * sizeof (long));
	out = (long *) malloc (count * sizeof (long));
	sol = (long *) malloc (count * sizeof (long));
	for (i = 0; i < count; i++)
	  {
	    *(in + i) = (rank == size - 1 ? i : 0);
	    *(sol + i) = 0;
	    *(out + i) = 0;
	  }
	MPC_Allreduce (in, out, count, MPC_LONG, MPC_BAND, comm);
	for (i = 0; i < count; i++)
	  {
	    if (*(out + i) != *(sol + i))
	      {
		errcnt++;
		fnderr++;
	      }
	  }
	if (fnderr)
	  fprintf (stderr,
		   "(%d) Error for type MPC_LONG and op MPC_BAND\n",
		   world_rank);
	free (in);
	free (out);
	free (sol);
      }


      {
	short *in, *out, *sol;
	int i, fnderr = 0;
	in = (short *) malloc (count * sizeof (short));
	out = (short *) malloc (count * sizeof (short));
	sol = (short *) malloc (count * sizeof (short));
	for (i = 0; i < count; i++)
	  {
	    *(in + i) = (rank == size - 1 ? i : 0);
	    *(sol + i) = 0;
	    *(out + i) = 0;
	  }
	MPC_Allreduce (in, out, count, MPC_SHORT, MPC_BAND, comm);
	for (i = 0; i < count; i++)
	  {
	    if (*(out + i) != *(sol + i))
	      {
		errcnt++;
		fnderr++;
	      }
	  }
	if (fnderr)
	  fprintf (stderr,
		   "(%d) Error for type MPC_SHORT and op MPC_BAND\n",
		   world_rank);
	free (in);
	free (out);
	free (sol);
      }


      {
	unsigned short *in, *out, *sol;
	int i, fnderr = 0;
	in = (unsigned short *) malloc (count * sizeof (unsigned short));
	out = (unsigned short *) malloc (count * sizeof (unsigned short));
	sol = (unsigned short *) malloc (count * sizeof (unsigned short));
	for (i = 0; i < count; i++)
	  {
	    *(in + i) = (rank == size - 1 ? i : 0);
	    *(sol + i) = 0;
	    *(out + i) = 0;
	  }
	MPC_Allreduce (in, out, count, MPC_UNSIGNED_SHORT, MPC_BAND, comm);
	for (i = 0; i < count; i++)
	  {
	    if (*(out + i) != *(sol + i))
	      {
		errcnt++;
		fnderr++;
	      }
	  }
	if (fnderr)
	  fprintf (stderr,
		   "(%d) Error for type MPC_UNSIGNED_SHORT and op MPC_BAND\n",
		   world_rank);
	free (in);
	free (out);
	free (sol);
      }


      {
	unsigned *in, *out, *sol;
	int i, fnderr = 0;
	in = (unsigned *) malloc (count * sizeof (unsigned));
	out = (unsigned *) malloc (count * sizeof (unsigned));
	sol = (unsigned *) malloc (count * sizeof (unsigned));
	for (i = 0; i < count; i++)
	  {
	    *(in + i) = (rank == size - 1 ? i : 0);
	    *(sol + i) = 0;
	    *(out + i) = 0;
	  }
	MPC_Allreduce (in, out, count, MPC_UNSIGNED, MPC_BAND, comm);
	for (i = 0; i < count; i++)
	  {
	    if (*(out + i) != *(sol + i))
	      {
		errcnt++;
		fnderr++;
	      }
	  }
	if (fnderr)
	  fprintf (stderr,
		   "(%d) Error for type MPC_UNSIGNED and op MPC_BAND\n",
		   world_rank);
	free (in);
	free (out);
	free (sol);
      }


      {
	unsigned long *in, *out, *sol;
	int i, fnderr = 0;
	in = (unsigned long *) malloc (count * sizeof (unsigned long));
	out = (unsigned long *) malloc (count * sizeof (unsigned long));
	sol = (unsigned long *) malloc (count * sizeof (unsigned long));
	for (i = 0; i < count; i++)
	  {
	    *(in + i) = (rank == size - 1 ? i : 0);
	    *(sol + i) = 0;
	    *(out + i) = 0;
	  }
	MPC_Allreduce (in, out, count, MPC_UNSIGNED_LONG, MPC_BAND, comm);
	for (i = 0; i < count; i++)
	  {
	    if (*(out + i) != *(sol + i))
	      {
		errcnt++;
		fnderr++;
	      }
	  }
	if (fnderr)
	  fprintf (stderr,
		   "(%d) Error for type MPC_UNSIGNED_LONG and op MPC_BAND\n",
		   world_rank);
	free (in);
	free (out);
	free (sol);
      }


      gerr += errcnt;
      if (errcnt > 0)
	printf ("Found %d errors on %d for MPC_BAND(0)\n", errcnt, rank);
      errcnt = 0;

/* Test BXOR */
      if (world_rank == 0)
	printf ("Testing MPC_BXOR...\n");

      {
	int *in, *out, *sol;
	int i, fnderr = 0;
	in = (int *) malloc (count * sizeof (int));
	out = (int *) malloc (count * sizeof (int));
	sol = (int *) malloc (count * sizeof (int));
	for (i = 0; i < count; i++)
	  {
	    *(in + i) = (rank == 1) * 0xf0;
	    *(sol + i) = (size > 1) * 0xf0;
	    *(out + i) = 0;
	  }
	MPC_Allreduce (in, out, count, MPC_INT, MPC_BXOR, comm);
	for (i = 0; i < count; i++)
	  {
	    if (*(out + i) != *(sol + i))
	      {
		errcnt++;
		fnderr++;
	      }
	  }
	if (fnderr)
	  fprintf (stderr,
		   "(%d) Error for type MPC_INT and op MPC_BXOR\n",
		   world_rank);
	free (in);
	free (out);
	free (sol);
      }


      {
	long *in, *out, *sol;
	int i, fnderr = 0;
	in = (long *) malloc (count * sizeof (long));
	out = (long *) malloc (count * sizeof (long));
	sol = (long *) malloc (count * sizeof (long));
	for (i = 0; i < count; i++)
	  {
	    *(in + i) = (rank == 1) * 0xf0;
	    *(sol + i) = (size > 1) * 0xf0;
	    *(out + i) = 0;
	  }
	MPC_Allreduce (in, out, count, MPC_LONG, MPC_BXOR, comm);
	for (i = 0; i < count; i++)
	  {
	    if (*(out + i) != *(sol + i))
	      {
		errcnt++;
		fnderr++;
	      }
	  }
	if (fnderr)
	  fprintf (stderr,
		   "(%d) Error for type MPC_LONG and op MPC_BXOR\n",
		   world_rank);
	free (in);
	free (out);
	free (sol);
      }


      {
	short *in, *out, *sol;
	int i, fnderr = 0;
	in = (short *) malloc (count * sizeof (short));
	out = (short *) malloc (count * sizeof (short));
	sol = (short *) malloc (count * sizeof (short));
	for (i = 0; i < count; i++)
	  {
	    *(in + i) = (rank == 1) * 0xf0;
	    *(sol + i) = (size > 1) * 0xf0;
	    *(out + i) = 0;
	  }
	MPC_Allreduce (in, out, count, MPC_SHORT, MPC_BXOR, comm);
	for (i = 0; i < count; i++)
	  {
	    if (*(out + i) != *(sol + i))
	      {
		errcnt++;
		fnderr++;
	      }
	  }
	if (fnderr)
	  fprintf (stderr,
		   "(%d) Error for type MPC_SHORT and op MPC_BXOR\n",
		   world_rank);
	free (in);
	free (out);
	free (sol);
      }


      {
	unsigned short *in, *out, *sol;
	int i, fnderr = 0;
	in = (unsigned short *) malloc (count * sizeof (unsigned short));
	out = (unsigned short *) malloc (count * sizeof (unsigned short));
	sol = (unsigned short *) malloc (count * sizeof (unsigned short));
	for (i = 0; i < count; i++)
	  {
	    *(in + i) = (rank == 1) * 0xf0;
	    *(sol + i) = (size > 1) * 0xf0;
	    *(out + i) = 0;
	  }
	MPC_Allreduce (in, out, count, MPC_UNSIGNED_SHORT, MPC_BXOR, comm);
	for (i = 0; i < count; i++)
	  {
	    if (*(out + i) != *(sol + i))
	      {
		errcnt++;
		fnderr++;
	      }
	  }
	if (fnderr)
	  fprintf (stderr,
		   "(%d) Error for type MPC_UNSIGNED_SHORT and op MPC_BXOR\n",
		   world_rank);
	free (in);
	free (out);
	free (sol);
      }


      {
	unsigned *in, *out, *sol;
	int i, fnderr = 0;
	in = (unsigned *) malloc (count * sizeof (unsigned));
	out = (unsigned *) malloc (count * sizeof (unsigned));
	sol = (unsigned *) malloc (count * sizeof (unsigned));
	for (i = 0; i < count; i++)
	  {
	    *(in + i) = (rank == 1) * 0xf0;
	    *(sol + i) = (size > 1) * 0xf0;
	    *(out + i) = 0;
	  }
	MPC_Allreduce (in, out, count, MPC_UNSIGNED, MPC_BXOR, comm);
	for (i = 0; i < count; i++)
	  {
	    if (*(out + i) != *(sol + i))
	      {
		errcnt++;
		fnderr++;
	      }
	  }
	if (fnderr)
	  fprintf (stderr,
		   "(%d) Error for type MPC_UNSIGNED and op MPC_BXOR\n",
		   world_rank);
	free (in);
	free (out);
	free (sol);
      }


      {
	unsigned long *in, *out, *sol;
	int i, fnderr = 0;
	in = (unsigned long *) malloc (count * sizeof (unsigned long));
	out = (unsigned long *) malloc (count * sizeof (unsigned long));
	sol = (unsigned long *) malloc (count * sizeof (unsigned long));
	for (i = 0; i < count; i++)
	  {
	    *(in + i) = (rank == 1) * 0xf0;
	    *(sol + i) = (size > 1) * 0xf0;
	    *(out + i) = 0;
	  }
	MPC_Allreduce (in, out, count, MPC_UNSIGNED_LONG, MPC_BXOR, comm);
	for (i = 0; i < count; i++)
	  {
	    if (*(out + i) != *(sol + i))
	      {
		errcnt++;
		fnderr++;
	      }
	  }
	if (fnderr)
	  fprintf (stderr,
		   "(%d) Error for type MPC_UNSIGNED_LONG and op MPC_BXOR\n",
		   world_rank);
	free (in);
	free (out);
	free (sol);
      }


      gerr += errcnt;
      if (errcnt > 0)
	printf ("Found %d errors on %d for MPC_BXOR(1)\n", errcnt, rank);
      errcnt = 0;


      {
	int *in, *out, *sol;
	int i, fnderr = 0;
	in = (int *) malloc (count * sizeof (int));
	out = (int *) malloc (count * sizeof (int));
	sol = (int *) malloc (count * sizeof (int));
	for (i = 0; i < count; i++)
	  {
	    *(in + i) = 0;
	    *(sol + i) = 0;
	    *(out + i) = 0;
	  }
	MPC_Allreduce (in, out, count, MPC_INT, MPC_BXOR, comm);
	for (i = 0; i < count; i++)
	  {
	    if (*(out + i) != *(sol + i))
	      {
		errcnt++;
		fnderr++;
	      }
	  }
	if (fnderr)
	  fprintf (stderr,
		   "(%d) Error for type MPC_INT and op MPC_BXOR\n",
		   world_rank);
	free (in);
	free (out);
	free (sol);
      }


      {
	long *in, *out, *sol;
	int i, fnderr = 0;
	in = (long *) malloc (count * sizeof (long));
	out = (long *) malloc (count * sizeof (long));
	sol = (long *) malloc (count * sizeof (long));
	for (i = 0; i < count; i++)
	  {
	    *(in + i) = 0;
	    *(sol + i) = 0;
	    *(out + i) = 0;
	  }
	MPC_Allreduce (in, out, count, MPC_LONG, MPC_BXOR, comm);
	for (i = 0; i < count; i++)
	  {
	    if (*(out + i) != *(sol + i))
	      {
		errcnt++;
		fnderr++;
	      }
	  }
	if (fnderr)
	  fprintf (stderr,
		   "(%d) Error for type MPC_LONG and op MPC_BXOR\n",
		   world_rank);
	free (in);
	free (out);
	free (sol);
      }


      {
	short *in, *out, *sol;
	int i, fnderr = 0;
	in = (short *) malloc (count * sizeof (short));
	out = (short *) malloc (count * sizeof (short));
	sol = (short *) malloc (count * sizeof (short));
	for (i = 0; i < count; i++)
	  {
	    *(in + i) = 0;
	    *(sol + i) = 0;
	    *(out + i) = 0;
	  }
	MPC_Allreduce (in, out, count, MPC_SHORT, MPC_BXOR, comm);
	for (i = 0; i < count; i++)
	  {
	    if (*(out + i) != *(sol + i))
	      {
		errcnt++;
		fnderr++;
	      }
	  }
	if (fnderr)
	  fprintf (stderr,
		   "(%d) Error for type MPC_SHORT and op MPC_BXOR\n",
		   world_rank);
	free (in);
	free (out);
	free (sol);
      }


      {
	unsigned short *in, *out, *sol;
	int i, fnderr = 0;
	in = (unsigned short *) malloc (count * sizeof (unsigned short));
	out = (unsigned short *) malloc (count * sizeof (unsigned short));
	sol = (unsigned short *) malloc (count * sizeof (unsigned short));
	for (i = 0; i < count; i++)
	  {
	    *(in + i) = 0;
	    *(sol + i) = 0;
	    *(out + i) = 0;
	  }
	MPC_Allreduce (in, out, count, MPC_UNSIGNED_SHORT, MPC_BXOR, comm);
	for (i = 0; i < count; i++)
	  {
	    if (*(out + i) != *(sol + i))
	      {
		errcnt++;
		fnderr++;
	      }
	  }
	if (fnderr)
	  fprintf (stderr,
		   "(%d) Error for type MPC_UNSIGNED_SHORT and op MPC_BXOR\n",
		   world_rank);
	free (in);
	free (out);
	free (sol);
      }


      {
	unsigned *in, *out, *sol;
	int i, fnderr = 0;
	in = (unsigned *) malloc (count * sizeof (unsigned));
	out = (unsigned *) malloc (count * sizeof (unsigned));
	sol = (unsigned *) malloc (count * sizeof (unsigned));
	for (i = 0; i < count; i++)
	  {
	    *(in + i) = 0;
	    *(sol + i) = 0;
	    *(out + i) = 0;
	  }
	MPC_Allreduce (in, out, count, MPC_UNSIGNED, MPC_BXOR, comm);
	for (i = 0; i < count; i++)
	  {
	    if (*(out + i) != *(sol + i))
	      {
		errcnt++;
		fnderr++;
	      }
	  }
	if (fnderr)
	  fprintf (stderr,
		   "(%d) Error for type MPC_UNSIGNED and op MPC_BXOR\n",
		   world_rank);
	free (in);
	free (out);
	free (sol);
      }


      {
	unsigned long *in, *out, *sol;
	int i, fnderr = 0;
	in = (unsigned long *) malloc (count * sizeof (unsigned long));
	out = (unsigned long *) malloc (count * sizeof (unsigned long));
	sol = (unsigned long *) malloc (count * sizeof (unsigned long));
	for (i = 0; i < count; i++)
	  {
	    *(in + i) = 0;
	    *(sol + i) = 0;
	    *(out + i) = 0;
	  }
	MPC_Allreduce (in, out, count, MPC_UNSIGNED_LONG, MPC_BXOR, comm);
	for (i = 0; i < count; i++)
	  {
	    if (*(out + i) != *(sol + i))
	      {
		errcnt++;
		fnderr++;
	      }
	  }
	if (fnderr)
	  fprintf (stderr,
		   "(%d) Error for type MPC_UNSIGNED_LONG and op MPC_BXOR\n",
		   world_rank);
	free (in);
	free (out);
	free (sol);
      }


      gerr += errcnt;
      if (errcnt > 0)
	printf ("Found %d errors on %d for MPC_BXOR(0)\n", errcnt, rank);
      errcnt = 0;


      {
	int *in, *out, *sol;
	int i, fnderr = 0;
	in = (int *) malloc (count * sizeof (int));
	out = (int *) malloc (count * sizeof (int));
	sol = (int *) malloc (count * sizeof (int));
	for (i = 0; i < count; i++)
	  {
	    *(in + i) = ~0;
	    *(sol + i) = 0;
	    *(out + i) = 0;
	  }
	MPC_Allreduce (in, out, count, MPC_INT, MPC_BXOR, comm);
	for (i = 0; i < count; i++)
	  {
	    if (*(out + i) != *(sol + i))
	      {
		errcnt++;
		fnderr++;
	      }
	  }
	if (fnderr)
	  fprintf (stderr,
		   "(%d) Error for type MPC_INT and op MPC_BXOR\n",
		   world_rank);
	free (in);
	free (out);
	free (sol);
      }


      {
	long *in, *out, *sol;
	int i, fnderr = 0;
	in = (long *) malloc (count * sizeof (long));
	out = (long *) malloc (count * sizeof (long));
	sol = (long *) malloc (count * sizeof (long));
	for (i = 0; i < count; i++)
	  {
	    *(in + i) = ~0;
	    *(sol + i) = 0;
	    *(out + i) = 0;
	  }
	MPC_Allreduce (in, out, count, MPC_LONG, MPC_BXOR, comm);
	for (i = 0; i < count; i++)
	  {
	    if (*(out + i) != *(sol + i))
	      {
		errcnt++;
		fnderr++;
	      }
	  }
	if (fnderr)
	  fprintf (stderr,
		   "(%d) Error for type MPC_LONG and op MPC_BXOR\n",
		   world_rank);
	free (in);
	free (out);
	free (sol);
      }


      {
	short *in, *out, *sol;
	int i, fnderr = 0;
	in = (short *) malloc (count * sizeof (short));
	out = (short *) malloc (count * sizeof (short));
	sol = (short *) malloc (count * sizeof (short));
	for (i = 0; i < count; i++)
	  {
	    *(in + i) = ~0;
	    *(sol + i) = 0;
	    *(out + i) = 0;
	  }
	MPC_Allreduce (in, out, count, MPC_SHORT, MPC_BXOR, comm);
	for (i = 0; i < count; i++)
	  {
	    if (*(out + i) != *(sol + i))
	      {
		errcnt++;
		fnderr++;
	      }
	  }
	if (fnderr)
	  fprintf (stderr,
		   "(%d) Error for type MPC_SHORT and op MPC_BXOR\n",
		   world_rank);
	free (in);
	free (out);
	free (sol);
      }


      {
	unsigned short *in, *out, *sol;
	int i, fnderr = 0;
	in = (unsigned short *) malloc (count * sizeof (unsigned short));
	out = (unsigned short *) malloc (count * sizeof (unsigned short));
	sol = (unsigned short *) malloc (count * sizeof (unsigned short));
	for (i = 0; i < count; i++)
	  {
	    *(in + i) = ~0;
	    *(sol + i) = 0;
	    *(out + i) = 0;
	  }
	MPC_Allreduce (in, out, count, MPC_UNSIGNED_SHORT, MPC_BXOR, comm);
	for (i = 0; i < count; i++)
	  {
	    if (*(out + i) != *(sol + i))
	      {
		errcnt++;
		fnderr++;
	      }
	  }
	if (fnderr)
	  fprintf (stderr,
		   "(%d) Error for type MPC_UNSIGNED_SHORT and op MPC_BXOR\n",
		   world_rank);
	free (in);
	free (out);
	free (sol);
      }


      {
	unsigned *in, *out, *sol;
	int i, fnderr = 0;
	in = (unsigned *) malloc (count * sizeof (unsigned));
	out = (unsigned *) malloc (count * sizeof (unsigned));
	sol = (unsigned *) malloc (count * sizeof (unsigned));
	for (i = 0; i < count; i++)
	  {
	    *(in + i) = ~0;
	    *(sol + i) = 0;
	    *(out + i) = 0;
	  }
	MPC_Allreduce (in, out, count, MPC_UNSIGNED, MPC_BXOR, comm);
	for (i = 0; i < count; i++)
	  {
	    if (*(out + i) != *(sol + i))
	      {
		errcnt++;
		fnderr++;
	      }
	  }
	if (fnderr)
	  fprintf (stderr,
		   "(%d) Error for type MPC_UNSIGNED and op MPC_BXOR\n",
		   world_rank);
	free (in);
	free (out);
	free (sol);
      }


      {
	unsigned long *in, *out, *sol;
	int i, fnderr = 0;
	in = (unsigned long *) malloc (count * sizeof (unsigned long));
	out = (unsigned long *) malloc (count * sizeof (unsigned long));
	sol = (unsigned long *) malloc (count * sizeof (unsigned long));
	for (i = 0; i < count; i++)
	  {
	    *(in + i) = ~0;
	    *(sol + i) = 0;
	    *(out + i) = 0;
	  }
	MPC_Allreduce (in, out, count, MPC_UNSIGNED_LONG, MPC_BXOR, comm);
	for (i = 0; i < count; i++)
	  {
	    if (*(out + i) != *(sol + i))
	      {
		errcnt++;
		fnderr++;
	      }
	  }
	if (fnderr)
	  fprintf (stderr,
		   "(%d) Error for type MPC_UNSIGNED_LONG and op MPC_BXOR\n",
		   world_rank);
	free (in);
	free (out);
	free (sol);
      }


      gerr += errcnt;
      if (errcnt > 0)
	printf ("Found %d errors on %d for MPC_BXOR(1-0)\n", errcnt, rank);
      errcnt = 0;

/* Test Maxloc */
      if (world_rank == 0)
	printf ("Testing MPC_MAXLOC...\n");

      {
	struct int_test
	{
	  int a;
	  int b;
	} *in, *out, *sol;
	int i, fnderr = 0;
	in = (struct int_test *) malloc (count * sizeof (struct int_test));
	out = (struct int_test *) malloc (count * sizeof (struct int_test));
	sol = (struct int_test *) malloc (count * sizeof (struct int_test));
	for (i = 0; i < count; i++)
	  {
	    (in + i)->a = (rank + i);
	    (in + i)->b = rank;
	    (sol + i)->a = (size - 1 + i);
	    (sol + i)->b = (size - 1);
	    (out + i)->a = 0;
	    (out + i)->b = -1;
	  }
	MPC_Allreduce (in, out, count, MPC_2INT, MPC_MAXLOC, comm);
	for (i = 0; i < count; i++)
	  {
	    if ((out + i)->a != (sol + i)->a || (out + i)->b != (sol + i)->b)
	      {
		errcnt++;
		fnderr++;
		fprintf (stderr, "(%d) Expected (%d,%d) got (%d,%d)\n",
			 world_rank, (int) ((sol + i)->a), (sol + i)->b,
			 (int) ((out + i)->a), (out + i)->b);
	      }
	  }
	if (fnderr)
	  fprintf (stderr,
		   "(%d) Error for type MPC_2INT and op MPC_MAXLOC (%d of %d wrong)\n",
		   world_rank, fnderr, count);
	free (in);
	free (out);
	free (sol);
      }


      {
	struct long_test
	{
	  long a;
	  int b;
	} *in, *out, *sol;
	int i, fnderr = 0;
	in = (struct long_test *) malloc (count * sizeof (struct long_test));
	out = (struct long_test *) malloc (count * sizeof (struct long_test));
	sol = (struct long_test *) malloc (count * sizeof (struct long_test));
	for (i = 0; i < count; i++)
	  {
	    (in + i)->a = (rank + i);
	    (in + i)->b = rank;
	    (sol + i)->a = (size - 1 + i);
	    (sol + i)->b = (size - 1);
	    (out + i)->a = 0;
	    (out + i)->b = -1;
	  }
	MPC_Allreduce (in, out, count, MPC_LONG_INT, MPC_MAXLOC, comm);
	for (i = 0; i < count; i++)
	  {
	    if ((out + i)->a != (sol + i)->a || (out + i)->b != (sol + i)->b)
	      {
		errcnt++;
		fnderr++;
		fprintf (stderr, "(%d) Expected (%d,%d) got (%d,%d)\n",
			 world_rank, (int) ((sol + i)->a), (sol + i)->b,
			 (int) ((out + i)->a), (out + i)->b);
	      }
	  }
	if (fnderr)
	  fprintf (stderr,
		   "(%d) Error for type MPC_LONG_INT and op MPC_MAXLOC (%d of %d wrong)\n",
		   world_rank, fnderr, count);
	free (in);
	free (out);
	free (sol);
      }


      {
	struct short_test
	{
	  short a;
	  int b;
	} *in, *out, *sol;
	int i, fnderr = 0;
	in =
	  (struct short_test *) malloc (count * sizeof (struct short_test));
	out =
	  (struct short_test *) malloc (count * sizeof (struct short_test));
	sol =
	  (struct short_test *) malloc (count * sizeof (struct short_test));
	for (i = 0; i < count; i++)
	  {
	    (in + i)->a = (rank + i);
	    (in + i)->b = rank;
	    (sol + i)->a = (size - 1 + i);
	    (sol + i)->b = (size - 1);
	    (out + i)->a = 0;
	    (out + i)->b = -1;
	  }
	MPC_Allreduce (in, out, count, MPC_SHORT_INT, MPC_MAXLOC, comm);
	for (i = 0; i < count; i++)
	  {
	    if ((out + i)->a != (sol + i)->a || (out + i)->b != (sol + i)->b)
	      {
		errcnt++;
		fnderr++;
		fprintf (stderr, "(%d) Expected (%d,%d) got (%d,%d)\n",
			 world_rank, (int) ((sol + i)->a), (sol + i)->b,
			 (int) ((out + i)->a), (out + i)->b);
	      }
	  }
	if (fnderr)
	  fprintf (stderr,
		   "(%d) Error for type MPC_SHORT_INT and op MPC_MAXLOC (%d of %d wrong)\n",
		   world_rank, fnderr, count);
	free (in);
	free (out);
	free (sol);
      }


      {
	struct float_test
	{
	  float a;
	  int b;
	} *in, *out, *sol;
	int i, fnderr = 0;
	in =
	  (struct float_test *) malloc (count * sizeof (struct float_test));
	out =
	  (struct float_test *) malloc (count * sizeof (struct float_test));
	sol =
	  (struct float_test *) malloc (count * sizeof (struct float_test));
	for (i = 0; i < count; i++)
	  {
	    (in + i)->a = (rank + i);
	    (in + i)->b = rank;
	    (sol + i)->a = (size - 1 + i);
	    (sol + i)->b = (size - 1);
	    (out + i)->a = 0;
	    (out + i)->b = -1;
	  }
	MPC_Allreduce (in, out, count, MPC_FLOAT_INT, MPC_MAXLOC, comm);
	for (i = 0; i < count; i++)
	  {
	    if ((out + i)->a != (sol + i)->a || (out + i)->b != (sol + i)->b)
	      {
		errcnt++;
		fnderr++;
		fprintf (stderr, "(%d) Expected (%d,%d) got (%d,%d)\n",
			 world_rank, (int) ((sol + i)->a), (sol + i)->b,
			 (int) ((out + i)->a), (out + i)->b);
	      }
	  }
	if (fnderr)
	  fprintf (stderr,
		   "(%d) Error for type MPC_FLOAT_INT and op MPC_MAXLOC (%d of %d wrong)\n",
		   world_rank, fnderr, count);
	free (in);
	free (out);
	free (sol);
      }


      {
	struct double_test
	{
	  double a;
	  int b;
	} *in, *out, *sol;
	int i, fnderr = 0;
	in =
	  (struct double_test *) malloc (count * sizeof (struct double_test));
	out =
	  (struct double_test *) malloc (count * sizeof (struct double_test));
	sol =
	  (struct double_test *) malloc (count * sizeof (struct double_test));
	for (i = 0; i < count; i++)
	  {
	    (in + i)->a = (rank + i);
	    (in + i)->b = rank;
	    (sol + i)->a = (size - 1 + i);
	    (sol + i)->b = (size - 1);
	    (out + i)->a = 0;
	    (out + i)->b = -1;
	  }
	MPC_Allreduce (in, out, count, MPC_DOUBLE_INT, MPC_MAXLOC, comm);
	for (i = 0; i < count; i++)
	  {
	    if ((out + i)->a != (sol + i)->a || (out + i)->b != (sol + i)->b)
	      {
		errcnt++;
		fnderr++;
		fprintf (stderr, "(%d) Expected (%d,%d) got (%d,%d)\n",
			 world_rank, (int) ((sol + i)->a), (sol + i)->b,
			 (int) ((out + i)->a), (out + i)->b);
	      }
	  }
	if (fnderr)
	  fprintf (stderr,
		   "(%d) Error for type MPC_DOUBLE_INT and op MPC_MAXLOC (%d of %d wrong)\n",
		   world_rank, fnderr, count);
	free (in);
	free (out);
	free (sol);
      }


      gerr += errcnt;
      if (errcnt > 0)
	printf ("Found %d errors on %d for MPC_MAXLOC\n", errcnt, rank);
      errcnt = 0;

/* Test minloc */
      if (world_rank == 0)
	printf ("Testing MPC_MINLOC...\n");


      {
	struct int_test
	{
	  int a;
	  int b;
	} *in, *out, *sol;
	int i, fnderr = 0;
	in = (struct int_test *) malloc (count * sizeof (struct int_test));
	out = (struct int_test *) malloc (count * sizeof (struct int_test));
	sol = (struct int_test *) malloc (count * sizeof (struct int_test));
	for (i = 0; i < count; i++)
	  {
	    (in + i)->a = (rank + i);
	    (in + i)->b = rank;
	    (sol + i)->a = i;
	    (sol + i)->b = 0;
	    (out + i)->a = 0;
	    (out + i)->b = -1;
	  }
	MPC_Allreduce (in, out, count, MPC_2INT, MPC_MINLOC, comm);
	for (i = 0; i < count; i++)
	  {
	    if ((out + i)->a != (sol + i)->a || (out + i)->b != (sol + i)->b)
	      {
		errcnt++;
		fnderr++;
		fprintf (stderr, "(%d) Expected (%d,%d) got (%d,%d)\n",
			 world_rank, (int) ((sol + i)->a), (sol + i)->b,
			 (int) ((out + i)->a), (out + i)->b);
	      }
	  }
	if (fnderr)
	  fprintf (stderr,
		   "(%d) Error for type MPC_2INT and op MPC_MINLOC (%d of %d wrong)\n",
		   world_rank, fnderr, count);
	free (in);
	free (out);
	free (sol);
      }


      {
	struct long_test
	{
	  long a;
	  int b;
	} *in, *out, *sol;
	int i, fnderr = 0;
	in = (struct long_test *) malloc (count * sizeof (struct long_test));
	out = (struct long_test *) malloc (count * sizeof (struct long_test));
	sol = (struct long_test *) malloc (count * sizeof (struct long_test));
	for (i = 0; i < count; i++)
	  {
	    (in + i)->a = (rank + i);
	    (in + i)->b = rank;
	    (sol + i)->a = i;
	    (sol + i)->b = 0;
	    (out + i)->a = 0;
	    (out + i)->b = -1;
	  }
	MPC_Allreduce (in, out, count, MPC_LONG_INT, MPC_MINLOC, comm);
	for (i = 0; i < count; i++)
	  {
	    if ((out + i)->a != (sol + i)->a || (out + i)->b != (sol + i)->b)
	      {
		errcnt++;
		fnderr++;
		fprintf (stderr, "(%d) Expected (%d,%d) got (%d,%d)\n",
			 world_rank, (int) ((sol + i)->a), (sol + i)->b,
			 (int) ((out + i)->a), (out + i)->b);
	      }
	  }
	if (fnderr)
	  fprintf (stderr,
		   "(%d) Error for type MPC_LONG_INT and op MPC_MINLOC (%d of %d wrong)\n",
		   world_rank, fnderr, count);
	free (in);
	free (out);
	free (sol);
      }


      {
	struct short_test
	{
	  short a;
	  int b;
	} *in, *out, *sol;
	int i, fnderr = 0;
	in =
	  (struct short_test *) malloc (count * sizeof (struct short_test));
	out =
	  (struct short_test *) malloc (count * sizeof (struct short_test));
	sol =
	  (struct short_test *) malloc (count * sizeof (struct short_test));
	for (i = 0; i < count; i++)
	  {
	    (in + i)->a = (rank + i);
	    (in + i)->b = rank;
	    (sol + i)->a = i;
	    (sol + i)->b = 0;
	    (out + i)->a = 0;
	    (out + i)->b = -1;
	  }
	MPC_Allreduce (in, out, count, MPC_SHORT_INT, MPC_MINLOC, comm);
	for (i = 0; i < count; i++)
	  {
	    if ((out + i)->a != (sol + i)->a || (out + i)->b != (sol + i)->b)
	      {
		errcnt++;
		fnderr++;
		fprintf (stderr, "(%d) Expected (%d,%d) got (%d,%d)\n",
			 world_rank, (int) ((sol + i)->a), (sol + i)->b,
			 (int) ((out + i)->a), (out + i)->b);
	      }
	  }
	if (fnderr)
	  fprintf (stderr,
		   "(%d) Error for type MPC_SHORT_INT and op MPC_MINLOC (%d of %d wrong)\n",
		   world_rank, fnderr, count);
	free (in);
	free (out);
	free (sol);
      }


      {
	struct float_test
	{
	  float a;
	  int b;
	} *in, *out, *sol;
	int i, fnderr = 0;
	in =
	  (struct float_test *) malloc (count * sizeof (struct float_test));
	out =
	  (struct float_test *) malloc (count * sizeof (struct float_test));
	sol =
	  (struct float_test *) malloc (count * sizeof (struct float_test));
	for (i = 0; i < count; i++)
	  {
	    (in + i)->a = (rank + i);
	    (in + i)->b = rank;
	    (sol + i)->a = i;
	    (sol + i)->b = 0;
	    (out + i)->a = 0;
	    (out + i)->b = -1;
	  }
	MPC_Allreduce (in, out, count, MPC_FLOAT_INT, MPC_MINLOC, comm);
	for (i = 0; i < count; i++)
	  {
	    if ((out + i)->a != (sol + i)->a || (out + i)->b != (sol + i)->b)
	      {
		errcnt++;
		fnderr++;
		fprintf (stderr, "(%d) Expected (%d,%d) got (%d,%d)\n",
			 world_rank, (int) ((sol + i)->a), (sol + i)->b,
			 (int) ((out + i)->a), (out + i)->b);
	      }
	  }
	if (fnderr)
	  fprintf (stderr,
		   "(%d) Error for type MPC_FLOAT_INT and op MPC_MINLOC (%d of %d wrong)\n",
		   world_rank, fnderr, count);
	free (in);
	free (out);
	free (sol);
      }


      {
	struct double_test
	{
	  double a;
	  int b;
	} *in, *out, *sol;
	int i, fnderr = 0;
	in =
	  (struct double_test *) malloc (count * sizeof (struct double_test));
	out =
	  (struct double_test *) malloc (count * sizeof (struct double_test));
	sol =
	  (struct double_test *) malloc (count * sizeof (struct double_test));
	for (i = 0; i < count; i++)
	  {
	    (in + i)->a = (rank + i);
	    (in + i)->b = rank;
	    (sol + i)->a = i;
	    (sol + i)->b = 0;
	    (out + i)->a = 0;
	    (out + i)->b = -1;
	  }
	MPC_Allreduce (in, out, count, MPC_DOUBLE_INT, MPC_MINLOC, comm);
	for (i = 0; i < count; i++)
	  {
	    if ((out + i)->a != (sol + i)->a || (out + i)->b != (sol + i)->b)
	      {
		errcnt++;
		fnderr++;
		fprintf (stderr, "(%d) Expected (%d,%d) got (%d,%d)\n",
			 world_rank, (int) ((sol + i)->a), (sol + i)->b,
			 (int) ((out + i)->a), (out + i)->b);
	      }
	  }
	if (fnderr)
	  fprintf (stderr,
		   "(%d) Error for type MPC_DOUBLE_INT and op MPC_MINLOC (%d of %d wrong)\n",
		   world_rank, fnderr, count);
	free (in);
	free (out);
	free (sol);
      }


      gerr += errcnt;
      if (errcnt > 0)
	printf ("Found %d errors on %d for MPC_MINLOC\n", errcnt, rank);
      errcnt = 0;

    }
  if (gerr > 0)
    {
      MPC_Comm_rank (MPC_COMM_WORLD, &rank);
      printf ("Found %d errors overall on %d\n", gerr, rank);
    }
  FreeComms (comms, ncomm);
  MPC_Finalize ();
  return 0;
}
