/*
 * This program performs some simple tests of the MPC_Bcast broadcast
 * functionality.
 */

#include "test.h"
#include "mpc.h"
#include <stdlib.h>

int
main (int argc, char **argv)
{
  int rank, size, ret, passed, i, *test_array;

  /* Set up MPC */
  MPC_Init (&argc, &argv);
  MPC_Comm_rank (MPC_COMM_WORLD, &rank);
  MPC_Comm_size (MPC_COMM_WORLD, &size);

  /* Setup the tests */
  Test_Init ("bcast", rank);
  test_array = (int *) malloc (size * sizeof (int));

  /* Perform the test - this operation should really be done
     with an allgather, but it makes a good test... */
  passed = 1;
  for (i = 0; i < size; i++)
    {
      if (i == rank)
	test_array[i] = i;
      MPC_Bcast (test_array, size, MPC_INT, i, MPC_COMM_WORLD);
      if (test_array[i] != i)
	passed = 0;
    }
  if (!passed)
    Test_Failed ("Simple Broadcast test");
  else
    {
      if (rank == 0)
	Test_Passed ("Simple Broadcast test");
    }

  /* Close down the tests */
  free (test_array);
  if (rank == 0)
    ret = Summarize_Test_Results ();
  else
    ret = 0;
  Test_Finalize ();

  /* Close down MPC */
  Test_Waitforall ();
  MPC_Finalize ();
  return ret;
}
