
#include <stdio.h>
#include "mpc.h"
#include "test.h"

#define TABLE_SIZE 2

int
main (int argc, char **argv)
{
  int rank, size;
  double a[TABLE_SIZE];
  struct
  {
    double a;
    int b;
  } in[TABLE_SIZE], out[TABLE_SIZE];
  int i;
  int errors = 0, toterrors;

  /* Initialize the environment and some variables */
  MPC_Init (&argc, &argv);
  MPC_Comm_rank (MPC_COMM_WORLD, &rank);
  MPC_Comm_size (MPC_COMM_WORLD, &size);

  /* Initialize the maxloc data */
  for (i = 0; i < TABLE_SIZE; i++)
    a[i] = 0;
  for (i = rank; i < TABLE_SIZE; i++)
    a[i] = (double) rank + 1.0;

  /* Copy data to the "in" buffer */
  for (i = 0; i < TABLE_SIZE; i++)
    {
      in[i].a = a[i];
      in[i].b = (double) rank;
    }

  /* Reduce it! */
  MPC_Reduce (in, out, TABLE_SIZE, MPC_DOUBLE_INT, MPC_MAXLOC, 0,
	      MPC_COMM_WORLD);
  MPC_Bcast (out, TABLE_SIZE, MPC_DOUBLE_INT, 0, MPC_COMM_WORLD);

  /* Check to see that we got the right answers */
  for (i = 0; i < TABLE_SIZE; i++)
    if (i % size == rank)
      if (out[i].b != rank)
	{
	  printf ("MAX (ranks[%d] = %d != %d\n", i, out[i].b, rank);
	  errors++;
	}

  /* Initialize the minloc data */
  for (i = 0; i < TABLE_SIZE; i++)
    a[i] = 0;
  for (i = rank; i < TABLE_SIZE; i++)
    a[i] = -(double) rank - 1.0;

  /* Copy data to the "in" buffer */
  for (i = 0; i < TABLE_SIZE; i++)
    {
      in[i].a = a[i];
      in[i].b = rank;
    }

  /* Reduce it! */
  MPC_Allreduce (in, out, TABLE_SIZE, MPC_DOUBLE_INT, MPC_MINLOC,
		 MPC_COMM_WORLD);

  /* Check to see that we got the right answers */
  for (i = 0; i < TABLE_SIZE; i++)
    if (i % size == rank)
      if (out[i].b != rank)
	{
	  printf ("MIN (ranks[%d] = %d != %d\n", i, out[i].b, rank);
	  errors++;
	}

  /* Finish up! */
  MPC_Allreduce (&errors, &toterrors, 1, MPC_INT, MPC_SUM, MPC_COMM_WORLD);
  if (toterrors)
    {
      if (errors)
	printf ("[%d] done with ERRORS(%d)!\n", rank, errors);
    }
  else
    {
      if (rank == 0)
	printf (" No Errors\n");
    }

  MPC_Finalize ();
  return errors;
}
