#include "mpc.h"
#include "test.h"

/* 
From: hook@nas.nasa.gov (Edward C. Hook)
 */

#include <stdlib.h>
#include <stdio.h>

#include <string.h>
#include <errno.h>
#ifndef EXIT_SUCCESS
#define EXIT_SUCCESS 0
#define EXIT_FAILURE 1
#endif

int
main (int argc, char *argv[])
{
  int rank, size;
  int chunk = 4096;
  int i;
  char *sb;
  char *rb;
  int status, gstatus;

  MPC_Init (&argc, &argv);
  MPC_Comm_rank (MPC_COMM_WORLD, &rank);
  MPC_Comm_size (MPC_COMM_WORLD, &size);

  for (i = 1; i < argc; ++i)
    {
      if (argv[i][0] != '-')
	continue;
      switch (argv[i][1])
	{
	case 'm':
	  chunk = atoi (argv[++i]);
	  break;
	default:
	  fprintf (stderr, "Unrecognized argument %s\n", argv[i]);
	  MPC_Abort (MPC_COMM_WORLD, EXIT_FAILURE);
	}
    }

  sb = (char *) malloc (size * chunk * sizeof (int));
  if (!sb)
    {
      perror ("can't allocate send buffer");
      MPC_Abort (MPC_COMM_WORLD, EXIT_FAILURE);
    }
  rb = (char *) malloc (size * chunk * sizeof (int));
  if (!rb)
    {
      perror ("can't allocate recv buffer");
      free (sb);
      MPC_Abort (MPC_COMM_WORLD, EXIT_FAILURE);
    }
  for (i = 0; i < size * chunk; ++i)
    {
      sb[i] = rank + 1;
      rb[i] = 0;
    }

  /* fputs("Before MPC_Alltoall\n",stdout); */

  /* This should really send MPC_CHAR, but since sb and rb were allocated
     as chunk*size*sizeof(int), the buffers are large enough */
  status = MPC_Alltoall (sb, chunk, MPC_INT, rb, chunk, MPC_INT,
			 MPC_COMM_WORLD);

  /* fputs("Before MPC_Allreduce\n",stdout); */
  MPC_Allreduce (&status, &gstatus, 1, MPC_INT, MPC_SUM, MPC_COMM_WORLD);

  /* fputs("After MPC_Allreduce\n",stdout); */
  if (rank == 0)
    {
      if (gstatus == 0)
	printf (" No Errors\n");
      else
	printf ("all_to_all returned %d\n", gstatus);
    }

  free (sb);
  free (rb);

  MPC_Finalize ();

  return (EXIT_SUCCESS);
}
