#include "mpc.h"
#include <stdio.h>
#include "test.h"

#define MAX_PROCESSES 10

int
main (int argc, char **argv)
{
  int rank, size, i, j;
  int table[MAX_PROCESSES][MAX_PROCESSES];
  int row[MAX_PROCESSES];
  int errors = 0;
  int participants;
  int displs[MAX_PROCESSES];
  int send_counts[MAX_PROCESSES];
  MPC_Comm testcomm;

  MPC_Init (&argc, &argv);
  MPC_Comm_rank (MPC_COMM_WORLD, &rank);
  MPC_Comm_size (MPC_COMM_WORLD, &size);

  /* A maximum of MAX_PROCESSES processes can participate */
  if (size > MAX_PROCESSES)
    participants = MAX_PROCESSES;
  else
    participants = size;
  /* Set the particpants so that it divides the MAX_PROCESSES */
  while (MAX_PROCESSES % participants)
    participants--;
  /* Create the communicator */
  MPC_Comm_split (MPC_COMM_WORLD, rank < participants, rank, &testcomm);
  if ((rank < participants))
    {
      int recv_count = MAX_PROCESSES;

      /* If I'm the root (process 0), then fill out the big table */
      /* and setup  send_counts and displs arrays */
      if (rank == 0)
	for (i = 0; i < participants; i++)
	  {
	    send_counts[i] = recv_count;
	    displs[i] = i * MAX_PROCESSES;
	    for (j = 0; j < MAX_PROCESSES; j++)
	      table[i][j] = i + j;
	  }

      /* Scatter the big table to everybody's little table */
      MPC_Scatterv (&table[0][0], send_counts, displs, MPC_INT,
		    &row[0], recv_count, MPC_INT, 0, testcomm);

      /* Now see if our row looks right */
      for (i = 0; i < MAX_PROCESSES; i++)
	if (row[i] != i + rank)
	  errors++;
    }

  Test_Waitforall ();
  MPC_Finalize ();
  if (errors)
    printf ("[%d] done with ERRORS(%d)!\n", rank, errors);
  return errors;
}
