#include "mpc.h"
#include <stdio.h>
#include <stdlib.h>
#include "test.h"

#define MAX_REQ 32
#define MAX_MSG_CNT 32
#define MAX_MSG 256
/* 
   This program tests a flood of data of short messages to test handling
   of both incoming messages and internal message queues
 */

void SetupData (int *, int, int);
void SetupRdata (int *, int);
int CheckData (int *, int, int, MPC_Status *);

#ifdef VERBOSE
static int verbose = 1;
#else
static int verbose = 0;
#endif


int
main (int argc, char **argv)
{
  MPC_Comm comm;
  MPC_Request r[MAX_REQ];
  MPC_Status s[MAX_REQ];
  int msgsize, maxmsg, root, i, j, size, rank, err = 0, msgcnt, toterr;
  int *sbuf, *rbuf;

  MPC_Init (&argc, &argv);

  comm = MPC_COMM_WORLD;

  MPC_Comm_size (comm, &size);
  MPC_Comm_rank (comm, &rank);

  if (size < 2)
    {
      printf ("This test requires at least 2 processors\n");
      MPC_Abort (comm, 1);
    }

  /* First, try large blocking sends to root */
  root = 0;

  maxmsg = MAX_MSG;
  msgsize = 128;
  msgcnt = MAX_MSG_CNT;
  if (rank == root && verbose)
    printf ("Blocking sends: ");
  while (msgsize <= maxmsg)
    {
      if (rank == root)
	{
	  if (verbose)
	    {
	      printf ("%d ", msgsize);
	      fflush (stdout);
	    }
	  rbuf = (int *) malloc (msgsize * sizeof (int));
	  if (!rbuf)
	    {
	      printf ("Could not allocate %d words\n", msgsize);
	      MPC_Abort (comm, 1);
	    }
	  for (i = 0; i < size; i++)
	    {
	      if (i == rank)
		continue;
	      for (j = 0; j < msgcnt; j++)
		{
		  SetupRdata (rbuf, msgsize);
		  MPC_Recv (rbuf, msgsize, MPC_INT, i, 2 * i, comm, s);
		  err += CheckData (rbuf, msgsize, 2 * i, s);
		}
	    }
	  free (rbuf);
	}
      else
	{
	  sbuf = (int *) malloc (msgsize * sizeof (int));
	  if (!sbuf)
	    {
	      printf ("Could not allocate %d words\n", msgsize);
	      MPC_Abort (comm, 1);
	    }
	  SetupData (sbuf, msgsize, 2 * rank);
	  for (j = 0; j < msgcnt; j++)
	    MPC_Send (sbuf, msgsize, MPC_INT, root, 2 * rank, comm);
	  free (sbuf);
	}
      msgsize *= 4;
    }
  if (rank == 0 && verbose)
    {
      printf ("\n");
      fflush (stdout);
    }

  /* Next, try unexpected messages with Isends */
  msgsize = 128;
  maxmsg = MAX_MSG;
  msgcnt = MAX_REQ;
  if (rank == root && verbose)
    printf ("Unexpected recvs: ");
  while (msgsize <= maxmsg)
    {
      if (rank == root)
	{
	  if (verbose)
	    {
	      printf ("%d ", msgsize);
	      fflush (stdout);
	    }
	  rbuf = (int *) malloc (msgsize * sizeof (int));
	  if (!rbuf)
	    {
	      printf ("Could not allocate %d words\n", msgsize);
	      MPC_Abort (comm, 1);
	    }
	  MPC_Barrier (comm);
	  for (i = 0; i < size; i++)
	    {
	      if (i == rank)
		continue;
	      for (j = 0; j < msgcnt; j++)
		{
		  SetupRdata (rbuf, msgsize);
		  MPC_Recv (rbuf, msgsize, MPC_INT, i, 2 * i, comm, s);
		  err += CheckData (rbuf, msgsize, 2 * i, s);
		}
	    }
	  free (rbuf);
	}
      else
	{
	  sbuf = (int *) malloc (msgsize * sizeof (int));
	  if (!sbuf)
	    {
	      printf ("Could not allocate %d words\n", msgsize);
	      MPC_Abort (comm, 1);
	    }
	  SetupData (sbuf, msgsize, 2 * rank);
	  for (j = 0; j < msgcnt; j++)
	    {
	      MPC_Isend (sbuf, msgsize, MPC_INT, root, 2 * rank, comm, &r[j]);
	    }
	  MPC_Barrier (comm);
	  MPC_Waitall (msgcnt, r, s);
	  free (sbuf);
	}
      msgsize *= 4;
    }
  if (rank == 0 && verbose)
    {
      printf ("\n");
      fflush (stdout);
    }

  /* Try large synchronous blocking sends to root */
  root = 0;

  msgsize = 128;
  maxmsg = MAX_MSG;
  if (rank == root && verbose)
    printf ("Synchronous sends: ");
  while (msgsize <= maxmsg)
    {
      if (rank == root)
	{
	  if (verbose)
	    {
	      printf ("%d ", msgsize);
	      fflush (stdout);
	    }
	  rbuf = (int *) malloc (msgsize * sizeof (int));
	  if (!rbuf)
	    {
	      printf ("Could not allocate %d words\n", msgsize);
	      MPC_Abort (comm, 1);
	    }
	  for (i = 0; i < size; i++)
	    {
	      if (i == rank)
		continue;
	      for (j = 0; j < msgcnt; j++)
		{
		  SetupRdata (rbuf, msgsize);
		  MPC_Recv (rbuf, msgsize, MPC_INT, i, 2 * i, comm, s);
		  err += CheckData (rbuf, msgsize, 2 * i, s);
		}
	    }
	  free (rbuf);
	}
      else
	{
	  sbuf = (int *) malloc (msgsize * sizeof (int));
	  if (!sbuf)
	    {
	      printf ("Could not allocate %d words\n", msgsize);
	      MPC_Abort (comm, 1);
	    }
	  SetupData (sbuf, msgsize, 2 * rank);
	  for (j = 0; j < msgcnt; j++)
	    MPC_Ssend (sbuf, msgsize, MPC_INT, root, 2 * rank, comm);
	  free (sbuf);
	}
      msgsize *= 4;
    }
  if (rank == 0 && verbose)
    {
      printf ("\n");
      fflush (stdout);
    }

  if (err)
    {
      printf ("!! found %d errors on processor %d\n", err, rank);
    }
  MPC_Allreduce (&err, &toterr, 1, MPC_INT, MPC_SUM, MPC_COMM_WORLD);

  if (rank == 0)
    {
      if (toterr == 0)
	printf (" No Errors\n");
      else
	printf ("!! found %d errors\n", toterr);
    }
  if (toterr)
    {
      printf ("!! found %d errors on processor %d\n", err, rank);
    }

  MPC_Finalize ();
  return toterr;
}

void
SetupData (sbuf, n, tag)
     int *sbuf, n, tag;
{
  int i;

  for (i = 0; i < n; i++)
    sbuf[i] = i;
}

int
CheckData (rbuf, n, tag, s)
     int *rbuf, n, tag;
     MPC_Status *s;
{
  int act_n, i;

  MPC_Get_count (s, MPC_INT, &act_n);
  if (act_n != n)
    {
      printf ("Received %d instead of %d ints\n", act_n, n);
      return 1;
    }
  for (i = 0; i < n; i++)
    {
      if (rbuf[i] != i)
	{
	  printf ("rbuf[%d] is %d, should be %d\n", i, rbuf[i], i);
	  return 1;
	}
    }
  return 0;
}

void
SetupRdata (rbuf, n)
     int *rbuf, n;
{
  int i;

  for (i = 0; i < n; i++)
    rbuf[i] = -(i + 1);
}
