/* This program comes from Bert Still, bert@h4p.llnl.gov 
   It caused problems for the T3D implementation.
 */
#include "test.h"
#include <mpc.h>
#include <stdio.h>

#define MESSAGE_TAG 8
#define MESSAGE_VALUE 6
#define MESSAGE_TYPE MPC_BYTE
#define MESSAGE_CTYPE char
static MESSAGE_CTYPE recv_msg[8];
static MESSAGE_CTYPE send_msg[8];

static MPC_Status recv_status;
static MPC_Status send_status[2];
static MPC_Request request[2];
static int complete[2];

/*------------------------------------------------------------------------*/

void fatal (int, char *);

void
fatal (rank, msg)
     int rank;
     char *msg;
{
  printf ("***FATAL** rank %d: %s\n", rank, msg);
  MPC_Abort (MPC_COMM_WORLD, 1);
  exit (1);
}

int
main (int argc, char *argv[])
{
  int size, rank;

  if (MPC_Init (&argc, &argv) != MPC_SUCCESS)
    fatal (-1, "MPC_Init failed");

  if (MPC_Comm_size (MPC_COMM_WORLD, &size) != MPC_SUCCESS)
    fatal (-1, "MPC_Comm_size failed");
  if (MPC_Comm_rank (MPC_COMM_WORLD, &rank) != MPC_SUCCESS)
    fatal (-1, "MPC_Comm_rank failed");
/*   if (size!=2) fatal(rank, "issend2 test requires -np 2\n"); */

  if (rank == 1)
    {
      if (MPC_Probe (MPC_ANY_SOURCE, MPC_ANY_TAG, MPC_COMM_WORLD,
		     &recv_status) != MPC_SUCCESS)
	fatal (rank, "MPC_Probe failed");
      if (recv_status.MPC_SOURCE != 0 || recv_status.MPC_TAG != MESSAGE_TAG)
	fatal (rank, "message source or tag wrong");
      if (MPC_Recv (recv_msg, 8, MESSAGE_TYPE,
		    recv_status.MPC_SOURCE, recv_status.MPC_TAG,
		    MPC_COMM_WORLD, &recv_status) != MPC_SUCCESS)
	fatal (rank, "MPC_Recv failed");
      if (recv_msg[0] == MESSAGE_VALUE)
	printf ("test completed successfully\n");
      else
	printf ("test failed: rank %d: got %d\n", rank, recv_msg[0]);

      fflush (stdout);

      if (recv_msg[0] != MESSAGE_VALUE)
	fatal (rank, "received message doesn't match sent message");

    }
  else if (rank == 0)
    {
      int n_complete;

      send_msg[0] = MESSAGE_VALUE;

      if (MPC_Issend (send_msg, 1, MESSAGE_TYPE, /*rank */ 1, MESSAGE_TAG,
		      MPC_COMM_WORLD, request) != MPC_SUCCESS)
	fatal (rank, "MPC_Issend failed");
      if (MPC_Waitsome (1, request, &n_complete, complete, send_status) !=
	  MPC_SUCCESS)
	fatal (rank, "MPC_Waitsome failed");
/*     if (request[0]!=MPC_REQUEST_NULL || n_complete!=1 || complete[0]!=0)  */
/* 	fatal(rank, "Waitsome result is wrong"); */
    }

  /* printf("rank %d: about to finalize\n", rank); */
  fflush (stdout);
  MPC_Finalize ();
  /*  printf("rank %d: finalize completed\n", rank); */
  fflush (stdout);
  return 0;
}
