/*
 * Program to test that the "synchronous send" semantics
 * of point to point communications in MPC is (probably) satisfied. 
 * This uses tests on the completions of the SENDS (unlike the MPC_Ssend
 * test) since the Issends return "immediately" but can not complete
 * until the matching receive begins.
 *
 * This program has been patterned off of "overtake.c"
 *
 *				William Gropp
 *				gropp@mcs.anl.gov
 */

#include <stdio.h>
#include "test.h"
#include "mpc.h"

#define SIZE 10000
/* Amount of time in seconds to wait for the receipt of the second Ssend
   message */
#define MAX_TIME 20
static int src = 1;
static int dest = 0;

/* Prototypes for picky compclers */
void Generate_Data (int *, int);

void
Generate_Data (buffer, buff_size)
     int *buffer;
     int buff_size;
{
  int i;

  for (i = 0; i < buff_size; i++)
    buffer[i] = i + 1;
}

int
main (int argc, char **argv)
{
  int rank;			/* My Rank (0 or 1) */
  int act_size = 1000;
  int flag;
  int buffer[SIZE];
  double t0;
  char *Current_Test = NULL;
  MPC_Status status;
  MPC_Request r1, r2;

  MPC_Init (&argc, &argv);
  MPC_Comm_rank (MPC_COMM_WORLD, &rank);

  /* This test depends on a working wtime.  Make a simple check */
/*     Current_Test = "Testing timer"; */
/*     t0 = MPC_Wtime(); */
/*     if (t0 == 0 && MPC_Wtime() == 0) { */
/* 	fprintf( stderr,  */
/* 		 "MPC_WTIME is returning 0; a working value is needed\n\ */
/* for this test.\n" ); */
/* 	Test_Failed(Current_Test); */
/* 	MPC_Abort( MPC_COMM_WORLD, 1 ); */
/*     } */
  /* Test that the timer increases */
/*     Current_Test = "Testing timer increases"; */
/*     for (flag=0; flag<1000000; flag++) { */
/* 	if (MPC_Wtime() > t0) break; */
/*     } */
/*     if (flag >= 1000000) { */
/* 	fprintf( stderr, "MPC_WTIME is not returning increasing values!\n" ); */
/* 	Test_Failed(Current_Test); */
/* 	MPC_Abort( MPC_COMM_WORLD, 1 ); */
/*     } */

  Current_Test = "Issend waits for recv";
  if (rank == src)
    {
      Test_Init ("issendtest", rank);
      Generate_Data (buffer, SIZE);
      MPC_Recv (buffer, 0, MPC_INT, dest, 0, MPC_COMM_WORLD, &status);
      MPC_Send (buffer, 0, MPC_INT, dest, 0, MPC_COMM_WORLD);
      MPC_Issend (buffer, act_size, MPC_INT, dest, 1, MPC_COMM_WORLD, &r1);
      MPC_Issend (buffer, act_size, MPC_INT, dest, 2, MPC_COMM_WORLD, &r2);
      t0 = MPC_Wtime ();
      flag = 0;
      while ((MPC_Wtime () - t0) < MAX_TIME)
	{
	  MPC_Test (&r1, &flag, &status);
	  if (flag)
	    {
	      Test_Failed (Current_Test);
	      break;
	    }
	}
      if (!flag)
	Test_Passed (Current_Test);
      MPC_Wait (&r2, &status);
      MPC_Sendrecv (buffer, 0, MPC_INT, dest, 13,
		    buffer, 0, MPC_INT, dest, 13, MPC_COMM_WORLD, &status);
      MPC_Wait (&r1, &status);
      Test_Waitforall ();
      MPC_Finalize ();
      {
	int rval = Summarize_Test_Results ();	/* Returns number of tests;
						   that failed */
	Test_Finalize ();
	return rval;
      }

    }
  else if (rank == dest)
    {
      /* Test 1 */
      MPC_Send (buffer, 0, MPC_INT, src, 0, MPC_COMM_WORLD);
      MPC_Recv (buffer, 0, MPC_INT, src, 0, MPC_COMM_WORLD, &status);
      MPC_Recv (buffer, act_size, MPC_INT, src, 2, MPC_COMM_WORLD, &status);
      MPC_Sendrecv (buffer, 0, MPC_INT, src, 13,
		    buffer, 0, MPC_INT, src, 13, MPC_COMM_WORLD, &status);
      MPC_Recv (buffer, act_size, MPC_INT, src, 1, MPC_COMM_WORLD, &status);

      Test_Waitforall ();
      MPC_Finalize ();
    }
  else
    {
      Test_Waitforall ();
      MPC_Finalize ();
/* 	fprintf(stderr, "*** This program uses exactly 2 processes! ***\n"); */
/* 	MPC_Abort( MPC_COMM_WORLD, 1 ); */
    }

  return 0;
}
