#include "test.h"
#include <mpc.h>
#include <stdio.h>
#include <stdlib.h>

#define MIN_MESSAGE_LENGTH 256
#define MAX_MESSAGE_LENGTH (16*1024*1024)
#define TAG1 1
#define TAG2 2
#define TAG3 3
#define TAG4 4
#define TAGSR 101

void Resetbuf (char *, int);
void Checkbuf (char *, int, MPC_Status *);

void
Resetbuf (char *buf, int len)
{
  int i;
  for (i = 0; i < len; i++)
    buf[i] = 0;
}

void
Checkbuf (char *buf, int len, MPC_Status * status)
{
  int count, i;
  int err = 0;
  char ival;

  MPC_Get_count (status, MPC_CHAR, &count);
  if (count != len)
    {
      fprintf (stderr, "Got len of %d but expected %d\n", count, len);
      err++;
    }
  ival = 0;
  for (i = 0; i < len; i++)
    {
      if (buf[i] != ival)
	{
	  err++;
	  fprintf (stderr,
		   "Found wrong value in buffer[%d] = %d, expected %d\n",
		   i, buf[i], ival);
	  if (err > 10)
	    break;
	}
      ival++;
    }
  if (err)
    MPC_Abort (MPC_COMM_WORLD, 1);
}

int
main (int argc, char *argv[])
{
  int msglen, i;
  int msglen_min = MIN_MESSAGE_LENGTH;
  int msglen_max = MAX_MESSAGE_LENGTH;
  int rank, poolsize, Master;
  char *sendbuf, *recvbuf;
  char ival;
  MPC_Request request;
  MPC_Status status;

  MPC_Init (&argc, &argv);
  MPC_Comm_size (MPC_COMM_WORLD, &poolsize);
  MPC_Comm_rank (MPC_COMM_WORLD, &rank);

/*     if(poolsize != 2) { */
/* 	printf("Expected exactly 2 MPC processes\n"); */
/* 	MPC_Abort( MPC_COMM_WORLD, 1 ); */
/*     } */
  Master = (rank == 0);

  if (rank == 0)
    printf ("Size (bytes)\n------------\n");
  for (msglen = msglen_min; msglen <= msglen_max; msglen *= 2)
    {

      sendbuf = malloc (msglen);
      recvbuf = malloc (msglen);
      if (sendbuf == NULL || recvbuf == NULL)
	{
	  printf ("Can't allocate %d bytes\n", msglen);
	  MPC_Abort (MPC_COMM_WORLD, 1);
	}

      ival = 0;
      for (i = 0; i < msglen; i++)
	{
	  sendbuf[i] = ival++;
	  recvbuf[i] = 0;
	}


      if (Master)
	printf ("%d\n", msglen);
      fflush (stdout);

      MPC_Barrier (MPC_COMM_WORLD);

      /* Send/Recv */
      if (rank == 0)
	MPC_Send (sendbuf, msglen, MPC_CHAR, 1, TAG1, MPC_COMM_WORLD);
      else if (rank == 1)
	{
	  Resetbuf (recvbuf, msglen);
	  MPC_Recv (recvbuf, msglen, MPC_CHAR, 0, TAG1, MPC_COMM_WORLD,
		    &status);
	  Checkbuf (recvbuf, msglen, &status);
	}

      MPC_Barrier (MPC_COMM_WORLD);

      /* Ssend/Recv */
      if (Master)
	MPC_Ssend (sendbuf, msglen, MPC_CHAR, 1, TAG2, MPC_COMM_WORLD);
      else if (rank == 1)
	{
	  Resetbuf (recvbuf, msglen);
	  MPC_Recv (recvbuf, msglen, MPC_CHAR, 0, TAG2, MPC_COMM_WORLD,
		    &status);
	  Checkbuf (recvbuf, msglen, &status);
	}

      MPC_Barrier (MPC_COMM_WORLD);

      /* Rsend/Recv */
      if (Master)
	{
	  MPC_Sendrecv (MPC_BOTTOM, 0, MPC_INT, 1, TAGSR,
			MPC_BOTTOM, 0, MPC_INT, 1, TAGSR,
			MPC_COMM_WORLD, &status);
	  MPC_Rsend (sendbuf, msglen, MPC_CHAR, 1, TAG3, MPC_COMM_WORLD);
	}
      else if (rank == 1)
	{
	  Resetbuf (recvbuf, msglen);
	  MPC_Irecv (recvbuf, msglen, MPC_CHAR, 0, TAG3, MPC_COMM_WORLD,
		     &request);
	  MPC_Sendrecv (MPC_BOTTOM, 0, MPC_INT, 0, TAGSR, MPC_BOTTOM, 0,
			MPC_INT, 0, TAGSR, MPC_COMM_WORLD, &status);
	  MPC_Wait (&request, &status);
	  Checkbuf (recvbuf, msglen, &status);
	}

      MPC_Barrier (MPC_COMM_WORLD);

      /* Isend/Recv - receive not ready */
      if (Master)
	{
	  MPC_Sendrecv (MPC_BOTTOM, 0, MPC_INT, 1, TAGSR,
			MPC_BOTTOM, 0, MPC_INT, 1, TAGSR,
			MPC_COMM_WORLD, &status);
	  MPC_Isend (sendbuf, msglen, MPC_CHAR, 1, TAG4, MPC_COMM_WORLD,
		     &request);
	  MPC_Wait (&request, &status);
	}
      else if (rank == 1)
	{
	  Resetbuf (recvbuf, msglen);
	  MPC_Sendrecv (MPC_BOTTOM, 0, MPC_INT, 0, TAGSR,
			MPC_BOTTOM, 0, MPC_INT, 0, TAGSR,
			MPC_COMM_WORLD, &status);
	  MPC_Recv (recvbuf, msglen, MPC_CHAR, 0, TAG4, MPC_COMM_WORLD,
		    &status);
	  Checkbuf (recvbuf, msglen, &status);
	}

      MPC_Barrier (MPC_COMM_WORLD);

      free (sendbuf);
      free (recvbuf);
    }

  if (rank == 0)
    printf ("Completed long message test\n");

  MPC_Finalize ();
  return 0;
}
