#include <stdio.h>
#include <stdlib.h>
#include "mpc.h"

#if defined(NEEDS_STDLIB_PROTOTYPES)
#include "protofix.h"
#endif

/*
   Test to make sure that nonblocking routines actually work.  This
   stresses them by sending large numbers of requests and receiving them
   piecemeal.
 */
int
main (int argc, char **argv)
{
  int count, tag, nsend, myid, np, rcnt, scnt, i, j, *buf;
  MPC_Status status;
  MPC_Request *rsend, *rrecv;

  MPC_Init (&argc, &argv);
  MPC_Comm_rank (MPC_COMM_WORLD, &myid);
  MPC_Comm_size (MPC_COMM_WORLD, &np);

  nsend = 3 * np;
  rsend = (MPC_Request *) malloc (nsend * sizeof (MPC_Request));
  rrecv = (MPC_Request *) malloc (nsend * sizeof (MPC_Request));
  if (!rsend || !rrecv)
    {
      fprintf (stderr, "Failed to allocate space for requests\n");
      MPC_Abort (MPC_COMM_WORLD, 1);
    }

  for (count = 1; count < 10000; count *= 2)
    {
      buf = (int *) malloc (count * sizeof (int));
      /* We'll send/recv from everyone */
      scnt = 0;
      rcnt = 0;
      for (j = 0; j < 3; j++)
	{
	  tag = j;
	  for (i = 0; i < np; i++)
	    {
	      if (i != myid)
		MPC_Isend (buf, count, MPC_INT, i, tag, MPC_COMM_WORLD,
			   &rsend[scnt++]);
	    }
	  for (i = 0; i < np; i++)
	    {
	      if (i != myid)
		MPC_Irecv (buf, count, MPC_INT, i, tag, MPC_COMM_WORLD,
			   &rrecv[rcnt++]);
	    }
	}
      /* In general, it would be better to use MPC_Waitall, but this should
         work as well */
      for (i = 0; i < rcnt; i++)
	{
	  MPC_Wait (&rrecv[i], &status);
	}
      for (i = 0; i < scnt; i++)
	{
	  MPC_Wait (&rsend[i], &status);
	}
      free (buf);

      if (myid == 0)
	{
	  printf
	    ("All processes wait for completion for count = %ld ints of data\n",
	     (long) count);
	  fflush (stdout);
	}
      MPC_Barrier (MPC_COMM_WORLD);
      if (myid == 0)
	{
	  printf ("All processes completed for count = %ld ints of data\n",
		  (long) count);
	  fflush (stdout);
	}
    }
  MPC_Finalize ();
  return 0;
}
