/*
 *  Test for null proc handling with blocking routines
 */


#include <stdio.h>
#include <mpc.h>

#if defined(NEEDS_STDLIB_PROTOTYPES)
#include "protofix.h"
#endif

int
main (int argc, char *argv[])
{
  int a[4];
  int i, nproc;
  int rank, right, left;
  MPC_Status st[2], sts[2];
  MPC_Request req[2];
  int count, errcnt = 0;

  /* start up */
  MPC_Init (&argc, &argv);
  MPC_Comm_size (MPC_COMM_WORLD, &nproc);
  MPC_Comm_rank (MPC_COMM_WORLD, &rank);

  /* set up processor chain (Apps should use Cart_create/shift) */
  left = (rank == 0) ? MPC_PROC_NULL : rank - 1;
  right = (rank == nproc - 1) ? MPC_PROC_NULL : rank + 1;

  /* initialize local matrix */
  /* globally: a[i] = i, i = 1 .. 2*nproc */
  /* locally : a[i] = 2*rank+i, i=1,2 */
  a[0] = -1;
  a[1] = 2 * rank + 1;
  a[2] = 2 * rank + 2;
  a[3] = -1;

  /* start all receives and sends */
  MPC_Isend (&a[1], 1, MPC_INT, left, 0, MPC_COMM_WORLD, &req[0]);
  MPC_Isend (&a[2], 1, MPC_INT, right, 1, MPC_COMM_WORLD, &req[1]);
  st[0].MPC_SOURCE = nproc;
  st[0].MPC_TAG = -1;
  st[1].MPC_SOURCE = nproc;
  st[1].MPC_TAG = -1;
  MPC_Recv (&a[0], 1, MPC_INT, left, 1, MPC_COMM_WORLD, &st[0]);
  MPC_Recv (&a[3], 1, MPC_INT, right, 0, MPC_COMM_WORLD, &st[1]);
  MPC_Waitall (2, req, sts);

  /* Test the end points */
  if (left == MPC_PROC_NULL)
    {
      if (st[0].MPC_TAG != MPC_ANY_TAG || st[0].MPC_SOURCE != MPC_PROC_NULL)
	{
	  errcnt++;
	  fprintf (stderr, "Incorrect null status for left\n");
	  if (st[0].MPC_SOURCE != MPC_PROC_NULL)
	    {
	      fprintf (stderr, "Source returned was %d but should be %d\n",
		       st[0].MPC_SOURCE, MPC_PROC_NULL);
	    }
	}
      MPC_Get_count (&st[0], MPC_INT, &count);
      if (count != 0)
	{
	  errcnt++;
	  fprintf (stderr, "Incorrect null status for left (count)\n");
	  fprintf (stderr, "Count was %d but should be 0\n", count);
	}
    }
  else if (right == MPC_PROC_NULL)
    {
      if (st[1].MPC_TAG != MPC_ANY_TAG || st[1].MPC_SOURCE != MPC_PROC_NULL)
	{
	  errcnt++;
	  fprintf (stderr, "Incorrect null status for right\n");
	  if (st[1].MPC_SOURCE != MPC_PROC_NULL)
	    {
	      fprintf (stderr, "Source returned was %d but should be %d\n",
		       st[1].MPC_SOURCE, MPC_PROC_NULL);
	    }
	}
      MPC_Get_count (&st[1], MPC_INT, &count);
      if (count != 0)
	{
	  errcnt++;
	  fprintf (stderr, "Incorrect null status for right (count)\n");
	  fprintf (stderr, "Count was %d but should be 0\n", count);
	}
    }

  /* Test results */
  if (left == MPC_PROC_NULL)
    {
      if (a[0] != -1)
	{
	  fprintf (stderr, "Expected -1, found %d in left partner\n", a[0]);
	  errcnt++;
	}
    }
  else
    {
      if (a[0] != 2 * left + 2)
	{
	  fprintf (stderr, "Expected %d, found %d in left partner\n",
		   2 * left + 2, a[0]);
	  errcnt++;
	}
    }

  if (right == MPC_PROC_NULL)
    {
      if (a[3] != -1)
	{
	  fprintf (stderr, "Expected -1, found %d in right partner\n", a[3]);
	  errcnt++;
	}
    }
  else
    {
      if (a[3] != 2 * right + 1)
	{
	  fprintf (stderr, "Expected %d, found %d in right partner\n",
		   2 * right + 1, a[3]);
	  errcnt++;
	}
    }


  i = errcnt;
  MPC_Allreduce (&i, &errcnt, 1, MPC_INT, MPC_SUM, MPC_COMM_WORLD);
  if (rank == 0)
    {
      if (errcnt > 0)
	{
	  printf ("Found %d errors in the run \n", errcnt);
	}
      else
	printf ("No errors in handling MPC_PROC_NULL\n");
    }

  /* clean up */
  MPC_Finalize ();
  return 0;
}
