#include <mpc.h>
#include <stdio.h>
#include "test.h"

/* 
 * Test that receives are done by relative rank, and that the status value
 * contains the relative rank
 */
int
main (int argc, char **argv)
{
  int rank, new_world_rank, size, order, errcnt = 0, i;
  int tmpcnt = 0;
  MPC_Comm new_world;
  MPC_Status s;

  MPC_Init (&argc, &argv);

  MPC_Comm_rank (MPC_COMM_WORLD, &rank);
  MPC_Comm_size (MPC_COMM_WORLD, &size);

  order = size - rank - 1;
  MPC_Comm_split (MPC_COMM_WORLD, 0, order, &new_world);

  MPC_Comm_rank (new_world, &new_world_rank);

  /* Make sure that the split worked correctly */
  if (new_world_rank != order)
    {
      errcnt++;
      fprintf (stderr, "Comm split did not properly order ranks!\n");
    }
  if (new_world_rank == 0)
    {
      MPC_Send (&tmpcnt, 1, MPC_INT, 1, 0, new_world);
      printf ("%d(%d): Sent message to: %d\n", new_world_rank, rank, 1);
    }
  else if (new_world_rank == 1)
    {
      MPC_Recv (&tmpcnt, 1, MPC_INT, 0, 0, new_world, &s);
      if (s.MPC_SOURCE != 0)
	{
	  errcnt++;
	  fprintf (stderr,
		   "Source incorrect in recv status (%d should be %d)\n",
		   s.MPC_SOURCE, 0);
	}

      printf
	("%d(%d): Recv message from: -> %d(%d) <- these 2 should equal\n",
	 new_world_rank, rank, 0, s.MPC_SOURCE);

    }

  MPC_Comm_free (&new_world);
  i = errcnt;
  MPC_Allreduce (&i, &errcnt, 1, MPC_INT, MPC_SUM, MPC_COMM_WORLD);
  if (errcnt > 0)
    {
      printf ("Found %d errors in the run\n", errcnt);
    }
  Test_Waitforall ();
  MPC_Finalize ();
  return 0;
}
