/*
 * Program to test that the "synchronous send" semantics
 * of point to point communications in MPC is (probably) satisfied. 
 * Two messages are send in one order; the destination uses MPC_Iprobe
 * to look for the SECOND message before doing a receive on the first.
 * To give a finite-termination, a fixed amount of time is used for
 * the Iprobe test.
 *
 * This program has been patterned off of "overtake.c"
 *
 *				William Gropp
 *				gropp@mcs.anl.gov
 */

#include <stdio.h>
#include "test.h"
#include "mpc.h"

#define SIZE 10000
/* Amount of time in seconds to wait for the receipt of the second Ssend
   message */
#define MAX_TIME 10
static int src = 0;
static int dest = 1;

/* Prototypes for picky compclers */
void Generate_Data (int *, int);

void
Generate_Data (int *buffer, int buff_size)
{
  int i;

  for (i = 0; i < buff_size; i++)
    buffer[i] = i + 1;
}

int
main (int argc, char **argv)
{
  int rank;			/* My Rank (0 or 1) */
  int act_size = 0;
  int flag, np, rval, i;
  int buffer[SIZE];
  double t0;
  char *Current_Test = NULL;
  MPC_Status status, status1, status2;
  int count1, count2;
  int sizes[4];

  MPC_Init (&argc, &argv);
  MPC_Comm_rank (MPC_COMM_WORLD, &rank);
  MPC_Comm_size (MPC_COMM_WORLD, &np);
/*     if (np != 2) { */
/*         fprintf(stderr, "*** This program uses exactly 2 processes! ***\n"); */
/*         MPC_Abort( MPC_COMM_WORLD, 1 ); */
/*         } */

  sizes[0] = 0;
  sizes[1] = 1;
  sizes[2] = 1000;
  sizes[3] = SIZE;
/*    for (i = 0; i < 4; i++ ) { */
  for (i = 1; i < 2; i++)
    {
      act_size = sizes[i];
      if (rank == src)
	{
	  Generate_Data (buffer, SIZE);
	  MPC_Recv (buffer, 0, MPC_INT, dest, 0, MPC_COMM_WORLD, &status);
	  MPC_Send (buffer, 0, MPC_INT, dest, 0, MPC_COMM_WORLD);
	  MPC_Ssend (buffer, act_size, MPC_INT, dest, 1, MPC_COMM_WORLD);
	  MPC_Ssend (buffer, act_size, MPC_INT, dest, 2, MPC_COMM_WORLD);

	}
      else if (rank == dest)
	{
	  Test_Init ("ssendtest", rank);
	  /* Test 1 */
	  Current_Test = "Ssend Test (Synchronous Send -> Normal Recieve)";
	  MPC_Send (buffer, 0, MPC_INT, src, 0, MPC_COMM_WORLD);
	  MPC_Recv (buffer, 0, MPC_INT, src, 0, MPC_COMM_WORLD, &status);
	  t0 = MPC_Wtime ();
	  flag = 0;
	  /* This test depends on a working wtime.  Make a simple check */
/* 	    if (t0 == 0 && MPC_Wtime() == 0) { */
/* 		fprintf( stderr,  */
/* 		 "MPC_WTIME is returning 0; a working value is needed\n\ */
/* for this test.\n" ); */
/* 		Test_Failed(Current_Test); */
/* 		MPC_Abort( MPC_COMM_WORLD, 1 ); */
/* 	    } */
	  while (MPC_Wtime () - t0 < MAX_TIME)
	    {
	      MPC_Iprobe (src, 2, MPC_COMM_WORLD, &flag, &status);
	      if (flag)
		{
		  Test_Failed (Current_Test);
		  break;
		}
	    }
	  if (!flag)
	    Test_Passed (Current_Test);
	  MPC_Recv (buffer, act_size, MPC_INT, src, 1, MPC_COMM_WORLD,
		    &status1);
	  MPC_Recv (buffer, act_size, MPC_INT, src, 2, MPC_COMM_WORLD,
		    &status2);

	  MPC_Get_count (&status1, MPC_INT, &count1);
	  MPC_Get_count (&status2, MPC_INT, &count2);
	  if (count1 != act_size)
	    {
	      fprintf (stdout,
		       "(1) Wrong count from recv of ssend: got %d (%d)\n",
		       count1, act_size);
	    }
	  if (status1.MPC_TAG != 1)
	    {
	      fprintf (stdout, "(1) Wrong tag from recv of ssend: got %d\n",
		       status1.MPC_TAG);
	    }
	  if (count2 != act_size)
	    {
	      fprintf (stdout,
		       "(2) Wrong count from recv of ssend: got %d (%d)\n",
		       count1, act_size);
	    }
	  if (status2.MPC_TAG != 2)
	    {
	      fprintf (stdout, "(2) Wrong tag from recv of ssend: got %d\n",
		       status2.MPC_TAG);
	    }

	}
    }

  Test_Waitforall ();
  rval = 0;
  if (rank == dest)
    {
      rval = Summarize_Test_Results ();	/* Returns number of tests;
					   that failed */
      Test_Finalize ();
    }
  MPC_Finalize ();
  return rval;
}
