/*
    third - test program that tests queueing by sending messages with various
            tags, receiving them in particular order.
 */

#include <stdio.h>
#include "mpc.h"
#include "test.h"
#ifdef HAVE_UNISTD_H
/* For sleep */
#include <unistd.h>
#endif

/* Define VERBOSE to get printed output */
int
main (int argc, char **argv)
{
  int rank, size, to, from, tag, count;
  int src, dest, waiter;
  int st_count;
#ifdef VERBOSE
  int st_source, st_tag;
#endif
  MPC_Status status;
  char data[100];
  MPC_Request rq[2];
  MPC_Status statuses[2];

  MPC_Init (&argc, &argv);
  MPC_Comm_rank (MPC_COMM_WORLD, &rank);
  MPC_Comm_size (MPC_COMM_WORLD, &size);
/*
    src  = size - 1;
    dest = 0;
 */
  src = 0;
  dest = size - 1;
  /* waiter = dest; *//* Receiver delays, so msgs unexpected */
  /* waiter = src;  *//* Sender delays, so recvs posted      */
  waiter = 10000;		/* nobody waits */

  if (rank == src)
    {
      if (waiter == src)
	sleep (10);
      to = dest;
      tag = 2001;
      sprintf (data, "First message, type 2001");
      count = strlen (data) + 1;
      MPC_Isend (data, count, MPC_CHAR, to, tag, MPC_COMM_WORLD, &rq[0]);
#ifdef VERBOSE
      printf ("%d sent :%s:\n", rank, data);
#endif
      tag = 2002;
      sprintf (data, "Second message, type 2002");
      count = strlen (data) + 1;
      MPC_Isend (data, count, MPC_CHAR, to, tag, MPC_COMM_WORLD, &rq[1]);
#ifdef VERBOSE
      printf ("%d sent :%s:\n", rank, data);
#endif
      MPC_Waitall (2, rq, statuses);
#ifdef VERBOSE
      printf ("%d all sent :%s:\n", rank, data);
#endif
    }
  else if (rank == dest)
    {
      if (waiter == dest)
	sleep (10);
      from = MPC_ANY_SOURCE;
      count = 100;

      tag = 2002;
      MPC_Recv (data, count, MPC_CHAR, from, tag, MPC_COMM_WORLD, &status);

      MPC_Get_count (&status, MPC_CHAR, &st_count);
      if (st_count != strlen ("Second message, type 2002") + 1)
	{
	  printf ("Received wrong length!\n");
	}
#ifdef VERBOSE
      st_source = status.MPC_SOURCE;
      st_tag = status.MPC_TAG;
      printf ("Status info: source = %d, tag = %d, count = %d\n",
	      st_source, st_tag, st_count);
      printf ("%d received :%s:\n", rank, data);
#endif
      tag = 2001;
      MPC_Recv (data, count, MPC_CHAR, from, tag, MPC_COMM_WORLD, &status);

      MPC_Get_count (&status, MPC_CHAR, &st_count);
      if (st_count != strlen ("First message, type 2001") + 1)
	{
	  printf ("Received wrong length!\n");
	}
#ifdef VERBOSE
      st_source = status.MPC_SOURCE;
      st_tag = status.MPC_TAG;
      printf ("Status info: source = %d, tag = %d, count = %d\n",
	      st_source, st_tag, st_count);
      printf ("%d received :%s:\n", rank, data);
#endif
    }
#ifdef VERBOSE
  printf ("Process %d exiting\n", rank);
#endif
  Test_Waitforall ();
  MPC_Finalize ();
  return 0;
}
