/*
  Test of waitall.  This makes sure that the requests in a wait can occur
  in any order.

  Run with 2 processes.
  */

#include <stdio.h>
#include <stdlib.h>
#include "mpc.h"


#if defined(NEEDS_STDLIB_PROTOTYPES)
#include "protofix.h"
#endif

void Paus (double);

void
Pause (double sec)
{
  double t1 = MPC_Wtime ();
  while (MPC_Wtime () - t1 < sec);
}

int
main (int argc, char **argv)
{
  int size, rank, flag, i;
  int *buf1, *buf2, cnt;
  double t0;
  MPC_Status statuses[2];
  MPC_Request req[2];

  MPC_Init (&argc, &argv);
  MPC_Comm_size (MPC_COMM_WORLD, &size);
  MPC_Comm_rank (MPC_COMM_WORLD, &rank);

/*   if (size < 2) { */
/*     printf( "This test requires at least 2 processors\n" ); */
/*     MPC_Abort( MPC_COMM_WORLD, 1 ); */
/*     return 1; */
/*   } */

  /* Large enough that almost certainly a rendezvous algorithm will be used
     by Issend.  buflimit.c will give you a more reliable value */
  cnt = 35000;

  /* Test:
     process 0                        process 1
     Irecv1
     Irecv2
     Sendrecv                         Sendrecv
     pause(2 sec)                     pause(2 sec)
     Issend2                          Waitall
     test(2) for 5 secs
     Ssend1
     Wait(2) if necessary

     If the test for Issend2 never succeeds, then the waitall appears to be
     waiting for req1 first.  By using Issend, we can keep the program from
     hanging.
   */
  buf1 = (int *) malloc (cnt * sizeof (int));
  buf2 = (int *) malloc (cnt * sizeof (int));
  if (!buf1 || !buf2)
    {
      printf ("Could not allocate buffers of size %d\n", cnt);
      MPC_Abort (MPC_COMM_WORLD, 1);
      return 1;
    }

  for (i = 0; i < cnt; i++)
    {
      buf1[i] = i;
      buf2[i] = i;
    }

  if (rank == 0)
    {
      MPC_Sendrecv (MPC_BOTTOM, 0, MPC_BYTE, size - 1, 3,
		    MPC_BOTTOM, 0, MPC_BYTE, size - 1, 3,
		    MPC_COMM_WORLD, &statuses[0]);
      Pause (2.0);
      MPC_Issend (buf2, cnt, MPC_INT, size - 1, 2, MPC_COMM_WORLD, &req[0]);
      t0 = MPC_Wtime ();
      flag = 0;
      while (t0 + 5.0 > MPC_Wtime () && !flag)
	MPC_Test (&req[0], &flag, &statuses[0]);
      MPC_Ssend (buf1, cnt, MPC_INT, size - 1, 1, MPC_COMM_WORLD);
      if (!flag)
	{
	  printf
	    ("*ERROR: MPC_Waitall appears to be waiting for requests in the order\n\
they appear in the request list\n");
	  MPC_Wait (&req[0], &statuses[0]);
	}
      else
	{
	  printf ("No errors\n");
	}
    }
  else if (rank == size - 1)
    {
      MPC_Irecv (buf1, cnt, MPC_INT, 0, 1, MPC_COMM_WORLD, &req[0]);
      MPC_Irecv (buf2, cnt, MPC_INT, 0, 2, MPC_COMM_WORLD, &req[1]);
      MPC_Sendrecv (MPC_BOTTOM, 0, MPC_BYTE, 0, 3,
		    MPC_BOTTOM, 0, MPC_BYTE, 0, 3, MPC_COMM_WORLD,
		    &statuses[0]);
      Pause (2.0);
      MPC_Waitall (2, req, statuses);
    }

  free (buf1);
  free (buf2);
  MPC_Finalize ();
  return 0;
}
